<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

class Initial extends AbstractMigration
{
    public $autoId = false;

    /**
     * Up Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-up-method
     * @return void
     */
    public function up()
    {
        $this->table('albums')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('favolabels_key', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addColumn('albumtitle', 'text', [
                'comment' => 'アルバム名',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('albumtitle_sort', 'text', [
                'comment' => 'アルバム名（読み）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('artist', 'text', [
                'comment' => '演奏者',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('artist_sort', 'text', [
                'comment' => '演奏者（読み）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('genre', 'text', [
                'comment' => 'ジャンル',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('path', 'text', [
                'comment' => 'アルバムへのパス',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('release_date', 'date', [
                'comment' => 'アルバムリリース年',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('songs_count', 'integer', [
                'comment' => 'カウンターキャッシュ',
                'default' => '0',
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('musicbrainz_albumid', 'string', [
                'comment' => 'mpdのmusicbrainz_albumid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addIndex(
                [
                    'favolabels_key',
                ]
            )
            ->create();

        $this->table('artistnames')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('kanji', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('yomi', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->create();

        $this->table('favolabels')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('colorcode', 'string', [
                'comment' => '色',
                'default' => null,
                'limit' => 10,
                'null' => true,
            ])
            ->addColumn('colorname', 'string', [
                'comment' => 'ラベル名称',
                'default' => null,
                'limit' => 64,
                'null' => true,
            ])
            ->addColumn('albums_count', 'integer', [
                'comment' => 'カウンターキャッシュ',
                'default' => '0',
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('label_key', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addIndex(
                [
                    'label_key',
                ],
                ['unique' => true]
            )
            ->create();

        $this->table('flags')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('scheduleon', 'tinyinteger', [
                'comment' => '1でスケジュール機能アクティブ',
                'default' => '0',
                'limit' => null,
                'null' => true,
            ])
            ->create();

        $this->table('genres')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('genre', 'string', [
                'comment' => 'ジャンル',
                'default' => null,
                'limit' => 200,
                'null' => true,
            ])
            ->create();

        $this->table('initial_letters')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('onechar', 'string', [
                'comment' => '頭文字',
                'default' => null,
                'limit' => 1,
                'null' => false,
            ])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addIndex(
                [
                    'onechar',
                ],
                ['unique' => true]
            )
            ->create();

        $this->table('mpdhosts')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('ipaddr', 'text', [
                'comment' => 'ip address or fqdn',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('nickname', 'text', [
                'comment' => '識別名',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('hostport', 'string', [
                'comment' => 'tcp port',
                'default' => null,
                'limit' => 10,
                'null' => true,
            ])
            ->addColumn('uname', 'string', [
                'comment' => '接続ユーザー名',
                'default' => null,
                'limit' => 45,
                'null' => true,
            ])
            ->addColumn('upass', 'string', [
                'comment' => '接続パスワード',
                'default' => null,
                'limit' => 45,
                'null' => true,
            ])
            ->create();

        $this->table('mpdstats')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('lastdates', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('artisits', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('songs', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('albums', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('yomifile', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->create();

        $this->table('netradios')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('radiourl', 'string', [
                'comment' => 'stream URL',
                'default' => null,
                'limit' => 200,
                'null' => true,
            ])
            ->addColumn('title', 'string', [
                'default' => null,
                'limit' => 200,
                'null' => true,
            ])
            ->create();

        $this->table('outputs')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('mpdhosts_id', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addColumn('devicename', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('deviceid', 'tinyinteger', [
                'comment' => 'デバイス番号',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('active', 'tinyinteger', [
                'default' => '0',
                'limit' => null,
                'null' => true,
            ])
            ->addIndex(
                [
                    'mpdhosts_id',
                ]
            )
            ->create();

        $this->table('schedules')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('title', 'string', [
                'comment' => 'タイトル',
                'default' => null,
                'limit' => 64,
                'null' => true,
            ])
            ->addColumn('playlistname', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('starttime', 'time', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('hvstop', 'tinyinteger', [
                'default' => '0',
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('stoptime', 'time', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('startmonth', 'tinyinteger', [
                'comment' => '月',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('startday', 'tinyinteger', [
                'comment' => '日',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('startyobi', 'string', [
                'comment' => '曜日指定（date(w)）の数値',
                'default' => null,
                'limit' => 20,
                'null' => true,
            ])
            ->addColumn('repeatflg', 'tinyinteger', [
                'comment' => '1でリピート',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('dostart', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('duration', 'time', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('onplay', 'tinyinteger', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->create();

        $this->table('songs')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('albums_id', 'integer', [
                'default' => null,
                'limit' => null,
                'null' => false,
                'signed' => false,
            ])
            ->addColumn('file', 'text', [
                'comment' => '曲へのフルパス',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('path', 'text', [
                'comment' => 'ディレクトリ名',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('title', 'text', [
                'comment' => '曲名',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('length', 'integer', [
                'comment' => '曲の長さ：秒',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('artist', 'text', [
                'comment' => 'artistタグ',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('artist_sort', 'text', [
                'comment' => 'ArtistSortタグの読み',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('composer', 'text', [
                'comment' => 'composerタグ',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('composer_sort', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('genre', 'text', [
                'comment' => 'genreタグ',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('track', 'integer', [
                'comment' => 'trackタグ（アルバム中の曲順）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('songdate', 'date', [
                'comment' => 'Dateタグ（年月日形式）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('albumtitle', 'text', [
                'comment' => 'Albumタグ（アルバムタイトル）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('albumtitle_sort', 'text', [
                'comment' => 'AlbumSortタグ（アルバムタイトル）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('album_artist', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('album_artist_sort', 'text', [
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('fileformat', 'string', [
                'comment' => 'サンプリングレート、量子化ビット数、チャンネル数',
                'default' => null,
                'limit' => 20,
                'null' => true,
            ])
            ->addColumn('last_modified', 'datetime', [
                'comment' => 'mpdのLast-Modified',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('performer', 'string', [
                'comment' => 'mpdのperformer',
                'default' => null,
                'limit' => 250,
                'null' => true,
            ])
            ->addColumn('conductor', 'string', [
                'comment' => 'mpdのconductor',
                'default' => null,
                'limit' => 250,
                'null' => true,
            ])
            ->addColumn('label', 'string', [
                'comment' => 'mpdのlabel',
                'default' => null,
                'limit' => 250,
                'null' => true,
            ])
            ->addColumn('musicbrainz_artistid', 'string', [
                'comment' => 'mpdのmusicbrainz_artistid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('musicbrainz_albumid', 'string', [
                'comment' => 'mpdのmusicbrainz_albumid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('musicbrainz_albumartistid', 'string', [
                'comment' => 'mpdのmusicbrainz_albumartistid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('musicbrainz_trackid', 'string', [
                'comment' => 'mpdのmusicbrainz_trackid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('musicbrainz_releasetrackid', 'string', [
                'comment' => 'mpdのmusicbrainz_releasetrackid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('musicbrainz_workid', 'string', [
                'comment' => 'mpdのmusicbrainz_workid',
                'default' => null,
                'limit' => 40,
                'null' => true,
            ])
            ->addColumn('sampling_frequency', 'decimal', [
                'comment' => '標本化数端数（単位はkHz、44.1,48,88.2,96,176.4,196くらい）',
                'default' => null,
                'null' => true,
                'precision' => 4,
                'scale' => 1,
            ])
            ->addColumn('quantization_bits', 'tinyinteger', [
                'comment' => '量子化ビット数（8,16,24,32）',
                'default' => null,
                'limit' => null,
                'null' => true,
            ])
            ->addColumn('channels', 'decimal', [
                'comment' => 'チャンネル数（1,2,4,5.1くらいか？）',
                'default' => null,
                'null' => true,
                'precision' => 2,
                'scale' => 1,
            ])
            ->addColumn('dsd_rate', 'string', [
                'comment' => 'dsdのレート',
                'default' => null,
                'limit' => 10,
                'null' => true,
            ])
            ->addIndex(
                [
                    'albums_id',
                ]
            )
            ->create();
    }

    /**
     * Down Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-down-method
     * @return void
     */
    public function down()
    {
        $this->table('albums')->drop()->save();
        $this->table('artistnames')->drop()->save();
        $this->table('favolabels')->drop()->save();
        $this->table('flags')->drop()->save();
        $this->table('genres')->drop()->save();
        $this->table('initial_letters')->drop()->save();
        $this->table('mpdhosts')->drop()->save();
        $this->table('mpdstats')->drop()->save();
        $this->table('netradios')->drop()->save();
        $this->table('outputs')->drop()->save();
        $this->table('schedules')->drop()->save();
        $this->table('songs')->drop()->save();
    }
}
