<?php

namespace App\Command;
use Cake\Console\Arguments;
use Cake\Console\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Datasource\ConnectionManager;
use Cake\Chronos\Chronos;
use Cake\Controller\ComponentRegistry;
use Cake\ORM\TableRegistry;
use App\Controller\Component\MpdComponent;
use Cake\I18n\FrozenDate;
use Cake\Log\Log;

class SetdataCommand extends Command
{

    private $mpdstats;
    private $songs;
    private $albums;
    private $genres;
    private $favolabels;
    private $artistnames;
    private $mpd;

    private $conio;

    public function initialize( ) : void
    {
        parent::initialize();

        mb_internal_encoding("UTF-8");
        mb_regex_encoding("UTF-8");

        $this->mpd = new MpdComponent(new ComponentRegistry());

        $this->mpdstats = TableRegistry::getTableLocator()->get('Mpdstats');
        $this->albums = TableRegistry::getTableLocator()->get('Albums');
        $this->songs = TableRegistry::getTableLocator()->get('Songs');
        $this->genres = TableRegistry::getTableLocator()->get('Genres');
        $this->favolabels = TableRegistry::getTableLocator()->get('Favolabels');
        $this->artistnames = TableRegistry::getTableLocator()->get('Artistnames');

        $this->conio = new ConsoleIo();

   }

    protected function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser
    {
        $parser = parent::buildOptionParser($parser);

        $parser
            ->addArgument('mode', [
                'help' => 'mpcbridge: 楽曲データベース更新ツール',
                'choices' => [ 'install', 'updatedb', 'setyomidb', 'updateyomi', 'cronmain', 'labelinit', ]
            ]);

        $parser->addOptions([
            'mode' =>
                [
                    'short' => 'm',
                    'help' => '実行コマンド'
                ],
        ]);


        return $parser;
    }

    public function execute(Arguments $args, ConsoleIo $io)
    {
        $mode = $args->getArgument('mode');
        $io->out( 'input is :'. $mode );

        switch ( $mode ) {
            case 'install':
                $io->out( 'installing...' );
                $this->install( );
                break;
            case 'updatedb':
                $io->out( 'updatedb' );
                $this->updatedb( );
                break;
            case 'setyomidb':
                $io->out( 'setyomidb' );
                $this->setyomidb( );
                break;
            case 'updateyomi':
                $io->out( 'updateyomi' );
                $this->updateyomi( );
                break;
            case 'cronmain':
                $io->out( 'cronmain' );
                $this->cronmain( );
                break;
            case 'labelinit':
                $io->out( 'initializing label data...' );
                $this->labelinit( );
                break;
        }

    }

    public function cronmain( )
    {

        $stat = $this->mpd->getStats( );

        $dbmatch = $ymmatch = true;

        $query = $this->mpdstats->find(  )->order( 'Mpdstats.id' );
        $crdb = $query->first( );
        if ( $crdb == null ) {
            $crdb = $this->mpdstats->newEmptyEntity();
            $dbmatch = false;
        }

        if ( is_null($crdb->lastdates) ) $dbmatch = false;
        if ( $dbmatch && ( $crdb->lastdates != $stat['stats']['db_update'] ) ) $dbmatch = false;
        $fn = CONFIG ."yomidata.txt";
        $ymtm = 0;

        if (is_file($fn)) $ymtm = filectime($fn);

        if ( $dbmatch && ( $ymtm > $crdb->yomifile ) ) $ymmatch = false;

        if ( ( ! $dbmatch ) || ( ! $ymmatch ) ) {
            $this->setyomidb( );
            $this->updatedb( );
            $this->updateyomi( );
        }
    }


    public function install( )
    {
        $this->albums->deleteAll( [  ] );
        $this->songs->deleteAll( [ ] );
        //  空のアルバムを作る
        $newalbums = $this->albums->newEntity( [
            'path' => 'none',
            'artist' => '',
            'artist_sort' => '',
            'albumtitle' => '',
            'genre' => '',
            'favolabels_key' => 1,
            'release_date' => '',
        ] );
        $rest = $this->albums->save( $newalbums );

        $this->setyomidb( );
        $this->updatedb( );
        $this->updateyomi( );
    }


    /*
     *  ラベルの初期化
     *  未使用にした（seedへ）
     */
    public function labelinit( )
    {

        $this->conio->out( 'start labelinit...' );
        $this->favolabels->deleteAll( [ 'id' > 0 ] );
        $data2 = [
            [ 'label_key'=>1, 'colorcode' => '#808080', 'colorname' => 'ラベルなし' ],
            [ 'label_key'=>2, 'colorcode' => '#DC143C', 'colorname' => '■ラベル-A' ],
            [ 'label_key'=>3, 'colorcode' => '#FF69B4', 'colorname' => '■ラベル-B' ],
            [ 'label_key'=>4, 'colorcode' => '#BA55D3', 'colorname' => '■ラベル-C' ],
            [ 'label_key'=>5, 'colorcode' => '#0000CD', 'colorname' => '■ラベル-D' ],
            [ 'label_key'=>6, 'colorcode' => '#6495ED', 'colorname' => '■ラベル-E' ],
            [ 'label_key'=>7, 'colorcode' => '#5F9EA0', 'colorname' => '■ラベル-F' ],
            [ 'label_key'=>8, 'colorcode' => '#00FF7F', 'colorname' => '■ラベル-G' ],
            [ 'label_key'=>9, 'colorcode' => '#B8860B', 'colorname' => '■ラベル-H' ],
            [ 'label_key'=>10, 'colorcode' => '#FFD700', 'colorname' => '■ラベル-I' ],
            [ 'label_key'=>11, 'colorcode' => '#FFDEAD', 'colorname' => '■ラベル-J' ],
            [ 'label_key'=>12, 'colorcode' => '#A52A2A', 'colorname' => '■ラベル-K' ],
        ];
        $entities = $this->favolabels->newEntities($data2);
//        Log::write( 'debug', '$entities : '. print_r( $entities, true ) );
        $result = $this->favolabels->saveMany($entities);
//        Log::write( 'debug', 'labelinit : '. print_r( $result, true ) );

        $this->albums->query( )->update( )->set( [ 'favolabels_key' => 1, 'modified = NOW()' ] )->execute();
    }

    /*
     *  mpd から取得した楽曲データベースを MySQLにコピーし、更新されなかった楽曲は削除
     */
    public function updatedb( )
    {

        $this->conio->out( 'start updatedb...' );

        mb_internal_encoding("UTF-8");
        mb_regex_encoding("UTF-8");

        //	全て削除
        $this->albums->deleteAll( [ 'id > ' => 0 ] );
        $this->songs->deleteAll( [ 'id > ' => 0 ] );

        $starttime = time( );
        $al = $this->_songandalbum( );

        $stat = $this->mpd->getStats( );
//        Log::write( 'debug', 'mpd->getStats : '. print_r( $stat, true ) );

        $query = $this->mpdstats->find(  )->order( 'Mpdstats.id' );
        $ds = $query->first( );
        if ( $ds == null ) {
            $savempdstats = $this->mpdstats->newEntity( [
                'lastdates' => $stat['stats']['db_update'],
                'artisits' => $stat['stats']['artists'],
                'songs' => $stat['stats']['songs'],
                'albums' => $stat['stats']['albums'],
            ] );
        } else {
            $savempdstats = $this->mpdstats->patchEntity( $ds, [
                'lastdates' => $stat['stats']['db_update'],
                'artisits' => $stat['stats']['artists'],
                'songs' => $stat['stats']['songs'],
                'albums' => $stat['stats']['albums'],
            ] );
        }
        $this->mpdstats->save( $savempdstats );

    }

    public function updateyomi( )
    {

        $this->conio->out( 'start updateyomi...' );

        mb_internal_encoding("UTF-8");
        mb_regex_encoding("UTF-8");

        //  アルバムの処理
        $calb = $csng = 0;
        $query = $this->albums
            ->find( 'list', [ 'keyField' => 'artist', 'valueField' => 'artist' ] )
            ->group( [ 'artist' ] )
            ->order( [ 'artist' => 'DESC' ] );
        $artist = $query->toArray();
        if ( ! empty( $artist ) ) {
//            Log::write( 'debug', 'updateyomi albums : '. date( 'Y-m-d H:i:s' ) );
            foreach ($artist as $one) {
                //	アーティスト名処理
//				$yomi = $one['Album']['artist'];
                //	The を取り除く
                $yomi = preg_replace( "/^(The\s)/i", "", $one );

                $oname = mb_ereg_replace( "[()＆（）・／、+/]", "&", $yomi );
                $match = mb_split( "[\s\&,;\:\(\)]+", $oname );

                foreach ( $match as $onef ) {
                    $onef = trim($onef);

                    $query_art = $this->artistnames->find( )->where( [ 'kanji' => $onef ] );
                    $ym = $query_art->first( );
                    if ( ( $ym != null ) && ( $ym->yomi != "" ) ) $yomi = str_replace( $onef, $ym->yomi, $yomi );

                }
                $yomi = mb_convert_kana( $yomi, "KCVa");

                if ( $one != $yomi ) {
                    $this->albums->query( )->update( )->set( [ 'artist_sort' => $yomi, ] )->where( [ 'artist' => $one ] )->execute();
                }

            }
        }

        //	楽曲
        $query_sg = $this->songs
            ->find( 'list', [ 'keyField' => 'artist', 'valueField' => 'artist' ] )
            ->group( [ 'artist' ] )
            ->order( [ 'artist' => 'ASC' ] );
        $song = $query_sg->toArray();
        if ( $song != null ) {
//		if ( $song !== false ) {
//            Log::write( 'debug', 'updateyomi songs : '. date( 'Y-m-d H:i:s' ) );
            foreach ($song as $one) {
                //	The を取り除く
                $yomi = preg_replace( "/^(The\s)/i", "", $one );

                $oname = mb_ereg_replace("[()＆（）・／、+/]", "&", $yomi );
                $match = mb_split("[\s\&,;\:\(\)（）・／]+", $oname );

                foreach ($match as $onef ) {
                    $onef = trim($onef);
                    $ym = $this->artistnames->find( )->where( [ 'kanji' => $onef ] )->first( );

                    if ( ( $ym != null ) && ( $ym->yomi != "" ) ) {
                        $yomi = str_replace($onef, $ym->yomi, $yomi );
                    }

                }
                $yomi = mb_convert_kana( $yomi, "KCVa");

                if ( $yomi != $one ) {
                    $this->songs->query( )->update( )->set( [ 'artist_sort' => $yomi, ] )->where( [ 'artist' => $one ] )->execute();
                }

            }
        }

    }

    public function setyomidb( )
    {

        $this->conio->out( 'start setyomidb...' );

        mb_internal_encoding("UTF-8");
        mb_regex_encoding("UTF-8");

        $artistNames = CONFIG ."yomidata.txt";

//		echo "yomidata file is ". $artistNames ."\n";
        if (is_file($artistNames) ) {
            $ymtm = filectime($artistNames);
//			echo "modified time is ". $ymtm ."\n";
            if ( ( $fp = fopen( $artistNames, "r") ) !== false ) {
                //	テーブルを空に

                $res_d = $this->artistnames->deleteAll( [ 1 => 1 ] );	//	全削除

                while (($buffer = fgets($fp, 4096)) !== false) {
                    $dat = explode("\t", $buffer);
                    if ( count($dat) > 1 ) {
                        $kj = $ym = '';
                        if ( strlen( trim( $dat[0] ) ) > 0 ) $kj = trim( $dat[0] );
                        if ( strlen( trim( $dat[1] ) ) > 0 ) $ym = trim( $dat[1] );
                        $kj = str_replace("　", "", str_replace(" ", "", $kj ) );
                        $ym = mb_convert_kana(str_replace("　", "", str_replace(" ", "", $ym ) ), "KCVa");

                        if ( ( $kj != '' ) && ( $ym != '' ) ) {
                            //	両方空欄じゃなかったらテーブルに保存
                            $one = $this->artistnames->newEntity( [ 'kanji' => $kj, 'yomi' => $ym ] );
                            $this->artistnames->save( $one );
                        }
                    }
                }

                $query = $this->mpdstats->find(  )->order( [ 'Mpdstats.id' => 'ASC'] );
                $ds = $query->first( );
                if ( $ds == null ) $ds = $this->mpdstats->newEmptyEntity();
                $ds->yomifile = $ymtm;
                $this->mpdstats->save($ds);

            }
        }
    }

    /*
     *  mpdのgetInfoコマンドで取得した楽曲情報を songs と akbums テーブルにコピー
     */
    function _songandalbum ( $dir = "" ) {

        $rt1 = $this->mpd->getInfo( $dir );	//	ルート階層を取得

        //  デフォルトのアルバムを
        $album_def = $this->albums->find( )->where( [ 'path' => 'none', ] )->first( );
        if ( isset( $album_def->id ) ) {
            $def_slb_id = $album_def->id;
        } else {
            $def_slb_id = 0;
        }
//        Log::write( 'debug', "_songandalbum : \n". $dir ."\n". print_r( $rt1, true ) );
        //	fileがあるなら、スキャンして保存
        $lastmodified = $albumArtist = $artistSort = $albumArtistSort = [ ];
        if ( isset( $rt1['Last-Modified'] ) ) $lastmodified = $rt1['Last-Modified'];
        if ( isset( $rt1['Label'] ) ) $labels = $rt1['Label'];
        if ( isset( $rt1['file'] ) ) {
            //  transaction
            $connection = ConnectionManager::get('default');
            //  ディレクトリ内のファイルをひっくるめて保存する


            //  ディレクトリ単位でtransaction
            $connection->begin();

            $songdatas = $albumdatas = $genres = [ ];
            $last_album = $last_path = '';
            $album_save_err = false;
            $last_album_id = 0;

            foreach ( $rt1['file'] as $fkey => $onesong ) {
                $file = $onesong['file'];	//	実ファイル
                $path = dirname( $onesong['file'] );
                //	曲名
                $fn = explode("/",$onesong['file']);
                if ( ! isset($onesong['Title']) ) {
                    $title = array_pop($fn);
                } else {
                    $title = $onesong['Title'];
                }
                //  演奏時間
                $length = '';
                if ( isset( $onesong['duration'] ) && ( strlen( $onesong['duration'] ) > 0 ) ) {
                    $length = intval( $onesong['duration'] );
                } elseif ( isset( $onesong['Time'] ) && ( strlen( $onesong['Time'] ) > 0 ) ) {
                    $length = intval( $onesong['Time'] );
                }
                //  演奏家 Artist と ArtistSort
                $artist = $artist_sort = '';
                if ( isset( $onesong['Artist'] ) && ( strlen( $onesong['Artist'] ) > 0 ) ) $artist = $onesong['Artist'];
                if ( isset( $onesong['ArtistSort'] ) && ( strlen( $onesong['ArtistSort'] ) > 0 ) ) {
                    $artist_sort = $onesong['ArtistSort'];
                } else {
                    $artist_sort = mb_convert_kana( $artist, "KCVa");
                }
                //  演奏家 AlbumArtist と AlbumArtistSort
                $album_artist = '';
                if ( isset( $onesong['AlbumArtist'] ) && ( strlen( $onesong['AlbumArtist'] ) > 0 ) ) $album_artist = $onesong['AlbumArtist'];
                if ( isset( $onesong['AlbumArtistSort'] ) ) {
                    $album_artist_sort = $onesong['AlbumArtistSort'];
                } else {
                    $album_artist_sort = $album_artist;
                }
                //  指揮者
                $conductor = '';
                if ( isset( $onesong['Conductor'] ) && ( strlen( $onesong['Conductor'] ) > 0 ) ) $conductor = $onesong['Conductor'];


                //  作曲家
                $composer = '';
                if ( isset( $onesong['Composer'] ) && ( strlen( $onesong['Composer'] ) > 0 ) ) $composer = $onesong['Composer'];
                $composer_sort = mb_convert_kana( $composer, "KCVa");
                //  ジャンル
                $genre = '';
                if ( isset( $onesong['Genre'] ) && ( strlen( $onesong['Genre'] ) > 0 ) ) $genre = $onesong['Genre'];
                $genres[ ] = $genre;
                // トラック
                $track = '';
                if ( isset( $onesong['Track'] ) && ( strlen( $onesong['Track'] ) > 0 ) ) $track = $onesong['Track'];

                //  楽曲のリリース年情報が年月日形式の場合は、年だけを取得
                $songdate = null;
                if ( isset( $onesong['Date'] ) && ( strlen( $onesong['Date'] ) > 0 ) ) {
                    //  年月日形式
                    if ( preg_match("/^(\d{4})-(\d{1,2})-(\d{1,2})/", $onesong['Date'], $match1 ) == 1 ) {
                        $songdate = new FrozenDate( $onesong['Date'] );
                    } elseif ( preg_match("/^(\d{4}).*/", $onesong['Date'], $match1 ) == 1 ) {
                        $songdate = new FrozenDate( $match1[1] ."-01-01" );
                    }
                }
                //  アルバムタイトル
                $albumtitle = '';
                if ( isset( $onesong['Album'] ) && ( strlen( $onesong['Album'] ) > 0 ) ) $albumtitle = $onesong['Album'];
                if ( isset( $onesong['AlbumSort'] ) && ( strlen( $onesong['AlbumSort'] ) > 0 ) ) {
                    $albumtitle_sort = $onesong['AlbumSort'];
                } else {
                    $albumtitle_sort = $albumtitle;
                }
                //  音声フォーマット
                $fileformat = '';
                if ( isset( $onesong['Format'] ) && ( strlen( $onesong['Format'] ) > 0 ) ) $fileformat = $onesong['Format'];
                $sampling_frequency = $quantization_bits = $channels = 0;
                $dsd_rate = '';
                $ret = preg_match( '/^(\d+):(.+):(\d+)$/', $fileformat, $vals );
                if ( ( $ret !== false ) && ( count( $vals ) == 4 ) ) {
                    $sampling_frequency = $vals[ 1 ] / 1000;
                    $channels = $vals[ 3 ];
                    if ( intval( $vals[ 2 ] ) > 0 ) $quantization_bits = intval( $vals[ 2 ] );
                }

                //  mpdのtag情報追加
                $last_modified = null;
                if ( isset( $lastmodified[ $fkey ] ) ) {
                    $last_modified = new Chronos($lastmodified[ $fkey ] );
                }

                if ( isset( $artistSort[ $fkey ] ) ) {
                    $artist_sort = $artistSort[ $fkey ];
                }

                $label = "";
                if ( isset( $labels[ $fkey ] ) ) {
                    $label = $labels[ $fkey ];
                }

                $onesong = [
                    'file' => $file,
                    'path' => $path,
                    'title' => $title,
                    'length' => $length,
                    'artist' => $artist, 'artist_sort' => $artist_sort,
                    'composer' => $composer, 'composer_sort' => $composer_sort, 'conductor' => $conductor,
                    'genre' => $genre, 'track' => $track, 'songdate' => $songdate,
                    'albumtitle' => $albumtitle, 'albumtitle_sort' => $albumtitle_sort,
                    'album_artist' => $album_artist, 'album_artist_sort' => $album_artist_sort,
                    'fileformat' => $fileformat, 'last_modified' => $last_modified,
                    'label' => $label,
                    'sampling_frequency' => $sampling_frequency, 'quantization_bits' => $quantization_bits, 'channels' => $channels, 'dsd_rate' => $dsd_rate,
                ];

                //  アルバム情報取得
                $rest = true;
                if ( ( $path != $last_path ) || ( $last_album != $albumtitle ) ) {
                    //  前回の処理とアルバムまたはパスが異なる
                    $quert_alb = $this->albums->find( )->where( [ 'path' => $path, 'albumtitle' => $albumtitle ] );
                    $alb = $quert_alb->first( );
                    if ( isset( $alb->id ) ) {
                        $onesong[ 'albums_id' ] = $alb->id;
                        $last_album_id = $alb->id;
                        $this->albums->updateAll( [ 'modified' => $quert_alb->func()->now( ) ], [ 'id' => $alb->id ] );

                    } else {
                        $onesong[ 'albums_id' ] = $def_slb_id;
                        $albmdata = [
                            'path' => $path,
                            'artist' => $artist,
                            'artist_sort' => $artist_sort,
                            'albumtitle' => $albumtitle,
                            'genre' => $genre,
                            'favolabels_key' => 1,
                            'release_date' => $songdate,
                        ];

                        $last_path = $path;
                        $last_album = $albumtitle;

                        $newalbums = $this->albums->newEntity( $albmdata );
                        if ( ! $newalbums->hasErrors ) {
                            if ( $rest = $this->albums->save( $newalbums ) ) {
                                $onesong[ 'albums_id' ] = $newalbums->id;
                                $last_album_id = $newalbums->id;
                            } else {
                                $album_save_err = true;
//                                $this->conio->out( 'album save error - entity '. print_r( $newalbums, true ) );
                                break;
                            }
                        }
                    }
                } else {
                    $onesong[ 'albums_id' ] = $last_album_id;
                }

                $songdatas[ ] = $onesong;

            }

            if ( $album_save_err ) {
                //  アルバムの保存のところでエラー
//                $this->conio->out( 'album_save error '. $path );
                $connection->rollback();
            } else {
                //  楽曲の保存

                /*
                 *  ジャンル検索
                 */
                $all_genres = array_unique( $genres );
                $t_genre = [ ];
                foreach ( $all_genres as $oneg ) {
                    if ( $this->genres->find( )->where( [ 'genre' => $oneg ] )->count( ) == 0 ) {
                        $t_genre[ ] = $oneg;
                        $this->conio->out( 'genre not in -- '. $oneg );
                    }
                }

                /*
                 *  存在しないジャンルがあったら保存
                 */
                $st1 = true;
                if ( count( $t_genre ) > 0 ) {
                    $save_genre = [ ];
                    foreach ( $t_genre as $item) {
                        $save_genre[ ] = [ 'genre' => $item ];
                    }
                    $ge_entities = $this->genres->newEntities( $save_genre );
                    $st2 = $this->genres->saveMany( $ge_entities );
                }

//                $this->conio->out( 'album_save success '. $path );
//                $this->conio->out( 'songdatas is '. print_r( $songdatas, true ) );
                $song_entities = $this->songs->newEntities( $songdatas );
                $st2 = $this->songs->saveMany( $song_entities );
                if ( $st1 && $st2 ) {
                    //  保存完了：コミット
//                    $this->conio->out( 'save '. $path );
                    $connection->commit();
                } else {
                    //  エラーなのでロールバック
//                    $this->conio->out( 'error '. $path );
                    $connection->rollback();
                }

            }


        }

        //	ディレクトリがあるなら、さらに中へ
        if ( isset( $rt1['directory'] ) ) {
            //	さらにディレクトリを掘る
            foreach ($rt1['directory'] as $val) {
//                $this->conio->out( 'next dir : '. $val );
                $this->_songandalbum($val);
            }
        }
        return;
    }

}
