<?php

namespace App\Controller;

use App\Controller\AppController;
use Cake\Event\Event;
use Cake\Core\Configure;
use Cake\ORM\TableRegistry;
use App\Controller\Component\RequestHandler;
use Cake\Network\Request;
use Cake\Log\Log;
use Cake\Routing\Router;

class IsearchController extends AppController
{

    private $coverUrl;
    private $alphabet;
    private $alphabetgrid;
    private $songs;
    private $albums;
    private $favolabels;
    private $initialLetters;
    private $sortkeys;

    private $doc_title;
    private $session;

    public function initialize(): void
    {
        parent::initialize();
        $this->session = $this->getRequest()->getSession();

        $this->albums = TableRegistry::getTableLocator()->get('Albums');
        $this->songs = TableRegistry::getTableLocator()->get('Songs');
        $this->favolabels = TableRegistry::getTableLocator()->get('Favolabels');
        $this->initialLetters = TableRegistry::getTableLocator()->get('InitialLetters');

        $this->coverUrl = Configure::read('CoverUrl');
        $this->alphabet = Configure::read('alphabet');
        $this->alphabetgrid = Configure::read('alphabetgrid');
        $this->sortkeys = Configure::read('Sortkey');
        $this->doc_title = "選曲";
    }

    public function beforeRender(\Cake\Event\EventInterface $event) {
        parent::beforeRender($event);

        $this->set( 'cover_url', $this->coverUrl );
        $this->set( 'alphabet', $this->alphabet );
        $this->set( 'alphabetgrid', $this->alphabetgrid );

        $labels = $this->_labels( );
        $this->set( 'labelcolor', $labels[0] );
        $this->set( 'labelname', $labels[1] );
        $this->set( 'title', $this->doc_title );
    }

    public function index( ) {
        $query = $this->songs->find( 'all' )->select(['artist_sort'])->group(['artist_sort']);
        $nums = $query->count();
        $this->set( 'allarts', $nums );
    }

    public function scsongtitle( $st = "" )
    {
        if ( $st == "" ) $this->redirect( "/" );
        $songnm = urldecode( trim($st) );
        $query = $this->songs->find( )->where( [ 'Songs.title' => $songnm ] );
        $query->order( [ 'Songs.artist_sort' => "ASC", "IF(Songs.songdate = NULL, '0000', Songs.songdate) ASC, Songs.track ASC", ] );
        $song_list = $query->all( );
        $allsong_num = $query->count( );
        $albums = $album_list = array( );
        if ( $allsong_num > 0 ) {
            $query2 = $this->songs->find( )->select( [ 'Songs.albums_id' ])->where( [ 'Songs.title' => $songnm ] );
            $album_list = $query2->group( [ 'Songs.albums_id' ] )->all( );
            if ( count($album_list)>0 ) {
                $aky = 0;
                foreach ($album_list as $aid ) {
                    $al = $this->albums->find( )->contain( ['Songs', 'Favolabels'] )->where( [ 'Albums.id' => $aid->albums_id ] )->first( );
                    if( ! empty( $al ) ) {
                        $albums[$aky] = $al;
                        $aky ++;
                    }
                }
            }

        } else {
            $this->redirect( "/" );
        }

        //  アーティスト名、前方一致
        $this->session->write( 'search', [ 'song_title' => $songnm, 'song_method' => 1 ] );

        $this->set('songnm', $songnm );
        $this->set('songnmur', $st );
        $this->set( 'allsong_num', $allsong_num );
        $this->set( 'albums', $albums );
        $this->set( 'song_list', $song_list );
        $this->set( 'album_list', $album_list );

    }

    public function scartist( $st = "" )
    {
        if ( $st == "" ) $this->redirect ( "/" );
        $artnam = urldecode( trim($st) );
        //	曲数
        $query = $this->songs->find( )->where( [ 'Songs.artist' => $artnam ] );
        $query->order( [ "IF(Songs.songdate = NULL, '0000', Songs.songdate) ASC, Songs.albums_id ASC, Songs.track ASC", ] );
        $allsong = $query->all( );
        $allsong_num = $query->count( );
        $albums = array( );
        if ( $allsong_num > 0 ) {
            $query2 = $this->songs->find( )->where( [ 'Songs.artist' => $artnam ] );
            $query2->select( [ 'Songs.albums_id' ]);
            $query2->group( [ 'Songs.albums_id' ] );
            $albumid = $query2->all( );
            $albumsu = $query2->count( );
            if ( $albumsu > 0 ) {
                $ids = [ ];
                foreach ($albumid as $aid ) {
                    $ids[ ] = $aid->albums_id;
                }
//                Log::write( 'debug', "albums_id s: ". print_r( $ids, true ) );
                $albums = $this ->albums->find( )
                    ->contain( ['Songs', 'Favolabels'] )
                    ->where( [ 'Albums.id IN' => $ids ] )
                    ->order( [ "IF(Albums.release_date = NULL, '0000', Albums.release_date) ASC" ] )
                    ->all( );
            }
        }

        //  アーティスト名、前方一致
        $this->session->write( 'search', [ 'artist_name' => $artnam, 'artist_method' => 1 ] );

        $this->set('artnam', $artnam );
        $this->set('artnamur', $st );
        $this->set('allsong', $allsong );
        $this->set('albums', $albums );

    }

    public function initial( $st = "", $srtkey = "s0" ) {

        if ( ! in_array( $srtkey, $this->sortkeys ) ) $srtkey = "s0";
        $st = mb_strtolower( mb_convert_kana( urldecode( $st ), "Hca" ) );
        $initsx = $inits = $this->_flatary( $this->alphabet );
        $initsx[ ] = 'その他';
        $allkana = mb_strtolower( mb_convert_kana( implode( "", $inits ), "KCa" ) );

        if ( ! in_array( $st, $initsx ) ) $this->redirect( [ 'action' => 'index' ] );

        $query = $this->songs->find( )->select( [ 'Songs.artist_sort', 'Songs.artist' ]);
        if ( $st == 'その他' ) {	//	読み仮名がアルファベット設定済み以外
            $notstr = "^[". $allkana ."]";
            $query	->where( [ 'NOT' => [ 'Songs.artist_sort' => "" ] ] )
                ->where( [ 'Songs.artist_sort NOT REGEXP' => '^(The )?'. $notstr ] )
                ->order( [ 'Songs.artist_sort' => "ASC" ] );

        } elseif ( $st == 't' ) {	//	Theで始まる連中
            $query	->where( [
                'OR' => [
                    [ 'Songs.artist_sort REGEXP' => '^The t' ],
                    [ 'AND' => [
                        [ 'Songs.artist_sort REGEXP' => '^t' ],
                        [ 'Songs.artist_sort NOT REGEXP' => '^The ' ]
                    ]
                    ],
                ]
            ] )
                ->order( [ 'Songs.artist_sort' => "ASC" ] );
        } else {	//	通常
            $kst = mb_convert_kana( $st, "C" );
            $query	->where( [
                'OR' => [
                    [ "Songs.artist_sort REGEXP" => "^(The )?{$kst}" ],
                    [ "Songs.artist REGEXP" => "^(The )?{$kst}" ],
                ]
            ] );
//			if ( preg_match( "/\w/", $kst ) == 0 ) {    //  何のためにこれをやったかわからん…
            $query->order( [ 'Songs.artist_sort' => "ASC" ] );
//			} else {
//				$query->order( [ 'Songs.artist' => "ASC" ] );
//			}
        }

        $query->group( [ 'Songs.artist' ] )->group( [ 'Songs.artist_sort' ] );

        $artistlist = $query->all( );
//        Log::write( 'debug', "initial: ". count( $artistlist ) );

        $this->set( 'allkana', $allkana );
        $this->set( 'initsx', $initsx );
        $this->set( 'artistlist', $artistlist );
        $this->set( 'st', $st );
        $this->set( 'srtkey', $srtkey );
        $this->doc_title .= "【{$st}】";

    }

    public function iniall( $srtkey = "s0" ) {

        $pgquery = $this->initialLetters->find( )->order( [ 'InitialLetters.id' => "ASC" ]);
        $inidata = $this->paginate( $pgquery, [ 'limit' => 1 ] );
        $inits = $this->_flatary( $this->alphabet );
        $allkana = mb_strtolower( mb_convert_kana( implode( "", $inits ), "KCa" ) );
//        Log::write( 'debug', '$inidata : ' . print_r( $inidata, true ) );
        if ( ! empty( $inidata ) ) {
            foreach ( $inidata as $one ) {
                $query = $this->songs->find( )->select( [ 'Songs.artist_sort', 'Songs.artist' ]);
                if ( $one->onechar == '-' ) {	//	読み仮名がアルファベット設定済み以外
                    $notstr = "^[". $allkana ."]";
                    $query	->where( [ 'NOT' => [ 'Songs.artist_sort' => "" ] ] )
                        ->where( [ 'Songs.artist_sort NOT REGEXP' => '^(The )?'. $notstr ] )
                        ->order( [ 'Songs.artist_sort' => "ASC" ] );

                } elseif ( $one->onechar == 't' ) {	//	Theで始まる連中
                    $query	->where( [
                        'OR' => [
                            [ 'Songs.artist_sort REGEXP' => '^The t' ],
                            [ 'AND' => [
                                [ 'Songs.artist_sort REGEXP' => '^t' ],
                                [ 'Songs.artist_sort NOT REGEXP' => '^The ' ]
                            ]
                            ],
                        ]
                    ] )
                        ->order( [ 'Songs.artist_sort' => "ASC" ] );
                } else {	//	通常
                    $kst = mb_convert_kana( $one->onechar, "C" );
                    $query	->where( [
                        'OR' => [
                            [ "Songs.artist_sort REGEXP" => "^(The )?{$kst}" ],
                            [ "Songs.artist REGEXP" => "^(The )?{$kst}" ],
                        ]
                    ] );
                    $query->order( [ 'Songs.artist_sort' => "ASC" ] );
                }
                $query->group( [ 'Songs.artist' ] )->group( [ 'Songs.artist_sort' ] );
                $artistlist = $query->all( );

            }
        }
        $this->set( 'charas', $inidata );
        $this->set( 'artistlist', $artistlist );

        $this->doc_title .= "（一覧）";

    }

    public function coverlist( $st = "", $srtkey = "s0" ) {
        $st = mb_strtolower( mb_convert_kana( urldecode( $st ), "Hca" ) );
        if ( ! in_array( $srtkey, $this->sortkeys ) ) $srtkey = "s0";

        $initsx = $inits = $this->_flatary( $this->alphabet );
        $initsx[ ] = 'その他';
        if ( ! ( in_array( $st, $initsx ) || ( $st == "iniall" ) ) ) $this->redirect( [ 'action'=>'index' ] );

        //  イニシャル絞り込み無し
        $iniall = false;
        if ( $st == 'iniall' ) $iniall = true;

        //	pagenation
        $query = $this->albums->find( )->select( [ 'Albums.id', 'Albums.albumtitle', 'Albums.artist', 'Albums.path', 'Albums.songs_count' ]);
        $query->matching( 'Songs' );
        if ( $st == 'その他' ) {
            $notstr = "^[". implode( "", $inits ) ."].*";
            $query->where( [
                'NOT' => [
                    'Songs.artist_sort REGEXP' => "^(The )?". mb_convert_kana( $notstr, "C" ),
                    'Songs.artist_sort' => ""
                ]
            ] );

        } elseif ( $st == 't' ) {
            $query->where( [
                'OR' => [
                    'Songs.artist_sort REGEXP' => "^The t",
                    'AND' => [
                        'Songs.artist_sort REGEXP' => "^t",
                        'Songs.artist_sort NOT REGEXP' => "^The ",
                    ]
                ]
            ] );

        } elseif ( ! $iniall ) {
            $kst = mb_convert_kana( $st, "C" );
            $query->where( [ 'Songs.artist_sort REGEXP' => "^(The )?{$kst}" ] );
        }

        switch ( $srtkey ) {
            case "s1":	//	年代降順
                $query->order( [ "IF(Albums.release_date = NULL, '0000', Albums.release_date) DESC", "IF(Albums.artist_sort = '', 'ヶ', Albums.artist_sort) ASC", ] );
                break;
            case "s2":	//	アルファベット昇順
                $query->order( [ "IF(Albums.artist_sort = '', 'ヶ', Albums.artist_sort) ASC", "IF(Albums.release_date = '', '9999', Albums.release_date) ASC", ] );
                break;
            case "s3":	//	アルファベット降順
                $query->order( [ "IF(Albums.artist_sort = '', ' ', Albums.artist_sort) DESC", "IF(Albums.release_date = '', '0000', Albums.release_date) DESC", ] );
                break;

            default:	//	年代昇順
                $query->order( [ "IF(Albums.release_date = NULL, '9999', Albums.release_date) ASC", "IF(Albums.artist_sort = '', 'ヶ', Albums.artist_sort) ASC", ] );
                break;
        }

        $query->group( [ 'Albums.id' ] );

        $this->set('albums', $this->paginate( $query, [ 'limit' => 36 ] ));
        $this->set( 'st', $st );

    }


    public function artist( $st = "", $artnam = "", $srtkey = "s0" ) {

        if ( ( $st == "" ) || ( $artnam == "" ) ) $this->redirect( [ 'action' => 'index' ] );

        $st = urldecode( trim($st) );
        $artnam =urldecode( trim($artnam) );

        //	曲情報
        $query = $this->songs->find( );
        $query->where( [ 'OR' => [ 'Songs.artist' => $artnam, 'Songs.artist_sort' => $artnam, ] ] );


        //	ソート順を作る
        switch ( $srtkey ) {
            case "s1":	//	年代降順
                $query->order( ["IF(Songs.songdate = NULL, '0000', Songs.songdate) DESC, Songs.albums_id ASC, Songs.track ASC" ] );
                break;
            case "s2":	//	アルファベット昇順
                $query->order( [ 'Songs.title' => "ASC", 'Songs.albums_id' => 'ASC', 'Songs.track' => 'ASC' ] );
                break;
            case "s3":	//	アルファベット降順
                $query->order( [ 'Songs.title' => "DESC", 'Songs.albums_id' => 'ASC', 'Songs.track' => 'ASC' ] );
                break;

            default:	//	年代昇順
                $query->order( [ "IF(Songs.songdate = NULL, '9999', Songs.songdate) ASC, Songs.albums_id ASC, Songs.track ASC", ] );
                break;
        }
        $allsong = $query->all( );
//        Log::write( 'debug', "{$artnam} : ". $query->sql() );
        $allsong_num = $query->count( );
//        Log::write( 'debug', '$allsong_num : ' .$allsong_num );

        if ( $allsong_num > 0 ) {
            //	含むアルバム
            $query2 = $this->songs->find( 'list', [ 'keyField' => 'albums_id', 'valueField' => 'albums_id' ] );
            $query2->where( [ 'OR' => [ 'Songs.artist' => $artnam, 'Songs.artist_sort' => $artnam, ] ] );
            $query2->group( [ 'Songs.albums_id' ] );
            $albumid = $query2->toArray();

            $query3 = $this->albums->find( )->contain( ['Songs', 'Favolabels'] );
            $query3->select( [ 'Albums.id', 'Albums.albumtitle', 'Albums.release_date', 'Albums.favolabels_key', 'Albums.genre', 'Albums.path', 'Favolabels.colorname', 'Albums.songs_count', 'Favolabels.colorcode' ] );
            $query3->where( [ 'Albums.id IN ' => $albumid, ] );
            //	ソート順を作る
            switch ( $srtkey ) {
                case "s1":	//	年代降順
                    $query3->order( [ "IF(Albums.release_date = NULL, '0000', Albums.release_date) DESC, Albums.albumtitle ASC", ] );
                    break;
                case "s2":	//	アルファベット昇順
                    $query3->order( [ 'Albums.albumtitle' => "ASC", "IF(Albums.release_date = NULL, '9999', Albums.release_date) ASC", ] );
                    break;
                case "s3":	//	アルファベット降順
                    $query3->order( [ 'Albums.albumtitle' => "DESC", "IF(Albums.release_date = NULL, '0000', Albums.release_date) DESC", ] );
                    break;

                default:	//	年代昇順
                    $query3->order( [ "IF(Albums.release_date = NULL, '9999', Albums.release_date) ASC, Albums.albumtitle ASC", ] );
                    break;
            }
            $all_albumdata = $query3->all( );
            $this->set( 'albumid', $albumid );
        } else {
            $all_albumdata = array( );
        }


        $this->set( 'allsong', $allsong );
        $this->set( 'allsong_num', $allsong_num );
        $this->set( 'st', $st );
        $this->set( 'artnam', $artnam );
        $this->set( 'srtkey', $srtkey );
        $this->set( 'all_albumdata', $all_albumdata );

    }

    function _labels ( ) {

        $lblall = $this->favolabels->find( )->order( [ 'Favolabels.label_key' => 'ASC' ] )->all( );
//		Log::write( 'debug', print_r( $lblall, TRUE ) );
        if ( ! empty( $lblall ) ) {
            $a1 = $a2 = array();
            foreach ( $lblall as $value ) {
                $a1[ $value->label_key ] = $value->colorcode;
                $a2[ $value->label_key ] = $value->colorname;
            }
            $retval = array(
                0 => $a1,
                1 => $a2
            );
        } else {
            $retval = array(
                0 => array( 1 => "808080" ),
                1 => array( 1 => "ラベルなし" )
            );
        }
        return $retval;
    }

    function _flatary ( $ary = null ) {
        if ( !is_array($ary) ) return array( );
        $retval = array( );
        foreach ($ary as $value) {
            if (is_array($value) ) {
                foreach ($value as $val ) {
                    $retval[ ] = $val;
                }
            } else {
                $retval[ ] = $value;
            }
        }
        return $retval;
    }

}
