<?php

/*

	Takeo Akiyama, Fourthgate Inc.

 */


namespace App\Controller;

use App\Controller\AppController;
use Cake\Event\Event;
use Cake\Core\Configure;
use Cake\ORM\TableRegistry;
use App\Controller\Component\RequestHandler;
use Cake\Controller\Component\RequestHandlerComponent;
use Cake\Network\Request;
use Cake\View\JsonView;
use Cake\Log\Log;

use App\Form\PlaylistForm;

class MpdapiController extends AppController
{

    private $seekTime = 10;
    private $netradio;

    const ON_PLAY = "play";
    const NOT_PLAY = "stop";
    const NOW_PAUSE = "pause";

    public function initialize( ): void
    {
        parent::initialize( );
        $this->loadComponent('RequestHandler');

        $this->netradio = TableRegistry::getTableLocator()->get('Netradios');
        $this->favolabels = TableRegistry::getTableLocator()->get('Favolabels');
        $this->albums = TableRegistry::getTableLocator()->get('Albums');

        $this->seekTime = Configure::read('Mpdplay.seektime');
    }

    /*
     * mpdの状態を返す
     */
    public function statmini( ) {

        $status = $this->Mpd->getStatus( );

        if ( isset($status['status'] ) ) {
            $setval = array('play'=>0);
            if ($status['status']['playlistlength']!=0) {
                $setval['play'] = 1;
                $setval[ 'current' ] = "(". ( $status['status']['song']+1 ) ."/". $status['status']['playlistlength'] .")";
            }
        } else {
            $setval = [ 'error' => true ];
        }
        $this->set('statmini', $setval );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'statmini');
    }

    /*
     *
     */
    public function currentstat( ) {
        $currentstatus = $this->Mpd->getStatus( );

        if ( isset($currentstatus['status'] ) ) {
            $currentsong['status'] = $currentstatus['status'];
        } else {
            $currentsong['status'] = [ ];
        }

        $this->set( 'status', $currentsong );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'status');
    }

    public function getcurrentstatus( ) {
        $currentsong = $this->Mpd->getCurrentSong( );
        $status = $this->Mpd->getStatus( );
        $noError = true;

        if ( isset( $status['status'] ) ) {
            $currentstatus = $status['status'];
            if ( isset($currentsong['file']) ) {
                //	プレイリストが存在
                $noplay = false;
                $fp = $currentsong['file'];
                //			$currentsong['filepath'] = urlencode( str_replace("/", "／／", dirname($fp) ) );
                //			$currentsong['filename'] = urlencode( str_replace("/", "／／", $fp ) );
                $currentsong['filepath'] = urlencode( dirname($fp ) );
                $currentsong['filename'] = urlencode( $fp );

                //	ネットラジオ
                if ( preg_match( "/http\:\/\//", $currentsong['file']) == 1 ) {
                    //	ストリーム
                    $currentsong['Album'] = $currentsong['Lartist'] = "";
                    if ( isset($currentsong['Name']) ) {
                        $currentsong['Artist'] = $currentsong['Name'];
                    } else {
                        $currentsong['Artist'] = "streaming(Internet radio)";
                    }
                    $currentsong['Genre'] = "";

                    $currentsong['timelength'] = 100;
                    $currentsong['timecurrent'] = 0;
                } else {
                    //	HDD上
                    $currentsong['Lartist'] = $currentsong['Lsong'] = "";
                    if ( ! isset($currentsong['Title']) ) {
                        $csg = explode( "/",$fp );
                        $currentsong['Title'] = array_pop( $csg );
                    }
                    if ( ! isset($currentsong['Album']) ) $currentsong['Album'] = "";
                    if ( ! isset($currentsong['Artist']) ) $currentsong['Artist'] = "";
                    if ( ! isset($currentsong['Genre']) ) $currentsong['Genre'] = "";
                    if ( $currentstatus['state'] == "stop" ) {
                        $currentsong['Title'] = "停止中";
                        $noplay = true;
                    } else {
                        if ( ! isset($currentsong['Title']) ) {
                            $csg = explode( "/", $fp );
                            $currentsong['Title'] = array_pop( $csg );
                        }
                    }
                    if ( isset($currentstatus['time']) ) {
                        list( $ct, $lg ) = explode( ":", $currentstatus['time'] );
                        $currentsong['timelength'] = $lg;
                        $currentsong['timecurrent'] = $ct;
                        $currentsong['songtime'] = Intval( $lg / 60 ) ."分". sprintf('%02d', ( $lg % 60 ) ) ."秒";
                    } else {
                        $currentsong['timelength'] = 100;
                        $currentsong['timecurrent'] = 0;
                        $currentsong['songtime'] = "";
                    }
                    if ( $currentsong['Artist'] != "" ) {
                        //					$currentsong['Lartist'] = urlencode( str_replace("/", "／／", $currentsong['Artist'] ) );
                        $currentsong['Lartist'] = urlencode( $currentsong['Artist'] );
                    }
                    if ( ( $currentsong['Title'] != "" ) && ( $currentsong['Title'] != "停止中" ) ) {
                        //					$currentsong['Lsong'] = urlencode( str_replace("/", "／／", $currentsong['Title'] ) );
                        $currentsong['Lsong'] = urlencode( $currentsong['Title'] );
                    }
                }
            } else {
                //	プレイリスト無し
                $currentsong['file'] = $currentsong['filepath'] = "";
                $currentsong['Album'] = $currentsong['Lartist'] = $currentsong['Artist'] = $currentsong['Genre'] = "";
                $currentsong['Title'] = $currentsong['Titledsp'] = "停止中";
                $currentsong['timelength'] = 100;
                $currentsong['timecurrent'] = 0;
                $noplay = true;
            }
            if ( isset($currentstatus['bitrate']) ) {
                if ( isset( $currentstatus['audio'] ) ) {
                    $audiostat = explode(":",$currentstatus['audio']);
                    if ( stripos( $audiostat[0], "dsd" ) === false ) {
                        //  PCM
                        $rate = $audiostat[0];
                        $depth = $audiostat[1];
                        $chnel = $audiostat[2];
                    } else {
                        //  DSD
                        $rate = $audiostat[0];
                        $depth = "dsd";
                        $chnel = $audiostat[1];
                    }
                } else {
                    $rate = $depth = $chnel = "";
                }
                if ( trim($depth) == "dsd" ) {
                    //	dsd
                    $currentsong['format'] = $rate;
                    $currentsong['bitrate'] = "";
                    $currentsong['audios'] = $rate;
                } else {
                    if ($currentstatus['bitrate']=="0") {
                        $currentsong['bitrate'] = "";
                    } else {
                        $currentsong['bitrate'] = number_format($currentstatus['bitrate']) ."kbps";
                    }
                    $ratedata = intval($rate/100)/10 ."kHz / ". $depth ."bit";
                    $currentsong['audios'] = $ratedata ." : ". number_format( $currentstatus['bitrate'] ) ."kbps";
                    $currentsong['format'] = intval($rate/100)/10 ."kHz / ". $depth ."bit";
                }
            } else {
                $currentsong['bitrate'] = "";
                $currentsong['audios'] = "";
                $currentsong['format'] = "";
            }

            if ( isset($currentstatus['state'] ) ) {
                $currentsong['state'] = $currentstatus['state'];
            } else {
                $currentsong['state'] = "";
            }
            if ( $currentstatus['random'] == 1 ) {
                $currentsong['random'] = 1;
            } else {
                $currentsong['random'] = 0;
            }
            $rpl = 0;
            if ( $currentstatus['repeat'] == 1 ) {
                $rpl = 1;
                if ( isset($status['single'] ) && ($status['single'][0]==1) ) $rpl = 2;
            }
            $currentsong['repeatmode'] = $rpl;

            if ( ! isset($currentsong['Composer']) ) $currentsong['Composer'] = "";
            if ( ! isset($currentsong['Date']) ) $currentsong['Date'] = "";

            //	次
            $currentsong['nTitle'] = "";
            $currentsong['nAlbum'] = "";
            $currentsong['nArtist'] = "";
            if ( ! $noplay ) {
                $playlist = $this->Mpd->getPlaylistInfo( $currentsong['Pos']+1 );
                if ( count($playlist) > 0 ) {
                    if ( isset($playlist[0]['Title']) ) {
                        $currentsong['nTitle'] = $playlist[0]['Title'];
                    } else {
                        if ( preg_match( "/http\:\/\//", $playlist[0]['file']) == 1 ) {
                            $currentsong['nTitle'] = "streaming(Internet radio)";
                        } elseif ( isset( $playlist[0]['file']) ) {
                            $wk = explode("/",$playlist[0]['file'] );
                            $currentsong['nTitle'] = array_pop( $wk );
                        } else {
                            $currentsong['nTitle'] = "unknown";
                        }
                    }
                    if ( isset($playlist[0]['Album']) ) {
                        $currentsong['nAlbum'] = $playlist[0]['Album'];
                    } else {
                        $currentsong['nAlbum'] = "";
                    }
                    if ( isset($playlist[0]['Artist']) ) {
                        $currentsong['nArtist'] = $playlist[0]['Artist'];
                    } else {
                        $currentsong['nArtist'] = "";
                    }
                    //				print_r($playlist);
                }
            }

            $this->set( 'currentsong', $currentsong );
        } else {
            $currentsong = [ 'error' => true ];
            $noError = true;
        }

        if ( $noError ) {
            $playlist_all = $this->Mpd->getPlaylistInfo( );
            $outary = array();
            foreach ($playlist_all as $value) {
                //	ネットラジオ
                if ( preg_match( "/http\:\/\//", $value['file']) == 1 ) {
                    $fn = explode("/",$value['file']);
                    if ( !isset($value['Title']) ) {
                        $title = array_pop($fn);
                        if ( $title == "" ) $title = "streaming(Internet radio)";
                    } else {
                        $title = $value['Title'];
                    }
                    if ( isset($value['Name'] ) ) {
                        $artist = $value['Name'];
                    } else {
                        $artist = "";
                    }
                } else {
                    //	ファイル群
                    $artist = ( isset( $value['Artist'] ) ) ? $value['Artist'] : NULL;
                    if ( ! isset($value['Title']) ) {
                        $wk = explode("/",$value['file'] );
                        $title = array_pop( $wk );
                    } else {
                        $title = $value['Title'];
                    }
                }
                $tm = (isset($value['Time'])) ? Intval( $value['Time'] / 60 ) .":". sprintf('%02d', ( $value['Time'] % 60 ) ) : NULL;
                $cs = ( isset($currentstatus['songid']) && ( $currentstatus['songid']==$value['Id'] ) ) ? 1 : 0;
                $album = ( isset( $value['Album'] ) ) ? $value['Album'] : NULL;
                $track = ( isset( $value['Track'] ) ) ? $value['Track'] : NULL;
                $outary[] = array(
                    'file' => $value['file'],
                    'Time' => $tm,
                    'Title' => $title,
                    'Artist' => $artist,
                    'Album' => $album,
                    //										'Genre' => $value['Genre'],
                    'Track' => $track,
                    //										'Disc' => $value['Disc'],
                    'Pos' => $value['Pos'],
                    'Id' => $value['Id'],
                    'nowplay' => $cs,
                );

            }
            $this->set( 'playlist', $outary );
        } else {
            $this->set( 'currentsong', [ 'error' => true ] );
            $this->set( 'playlist', [ 'error' => true ] );
        }

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', ['currentsong','playlist']);
    }

    public function currentsong( ) {
        $currentsong = $this->Mpd->getCurrentSong( );
        $status = $this->Mpd->getStatus( );

        if ( isset( $status['status'] ) ) {
            $currentstatus = $status['status'];
            if ( isset($currentsong['file']) ) {
                //	プレイリストが存在
                $noplay = false;
                $fp = $currentsong['file'];

                $currentsong['filepath'] = urlencode( dirname($fp ) );
                $currentsong['filename'] = urlencode( $fp );

                //	ネットラジオ
                if ( preg_match( "/http\:\/\//", $currentsong['file']) == 1 ) {
                    //	ストリーム
                    $currentsong['Album'] = $currentsong['Lartist'] = "";
                    if ( isset($currentsong['Name']) ) {
                        $currentsong['Artist'] = $currentsong['Name'];
                    } else {
                        $currentsong['Artist'] = "streaming(Internet radio)";
                    }
                    $currentsong['Genre'] = "";

                    $currentsong['timelength'] = 100;
                    $currentsong['timecurrent'] = 0;
                } else {
                    //	HDD上
                    $currentsong['Lartist'] = $currentsong['Lsong'] = "";
                    if ( ! isset($currentsong['Title']) ) {
                        $csg = explode( "/",$fp );
                        $currentsong['Title'] = array_pop( $csg );
                    }
                    if ( ! isset($currentsong['Album']) ) $currentsong['Album'] = "";
                    if ( ! isset($currentsong['Artist']) ) $currentsong['Artist'] = "";
                    if ( ! isset($currentsong['Genre']) ) $currentsong['Genre'] = "";
                    if ( $currentstatus['state'] == "stop" ) {
                        $currentsong['Title'] = "停止中";
                        $noplay = true;
                    } else {
                        if ( ! isset($currentsong['Title']) ) {
                            $csg = explode( "/", $fp );
                            $currentsong['Title'] = array_pop( $csg );
                        }
                    }
                    if ( isset($currentstatus['time']) ) {
                        list( $ct, $lg ) = explode( ":", $currentstatus['time'] );
                        $currentsong['timelength'] = $lg;
                        $currentsong['timecurrent'] = $ct;
                        $currentsong['songtime'] = Intval( $lg / 60 ) ."分". sprintf('%02d', ( $lg % 60 ) ) ."秒";
                    } else {
                        $currentsong['timelength'] = 100;
                        $currentsong['timecurrent'] = 0;
                        $currentsong['songtime'] = "";
                    }
                    if ( $currentsong['Artist'] != "" ) {
                        //					$currentsong['Lartist'] = urlencode( str_replace("/", "／／", $currentsong['Artist'] ) );
                        $currentsong['Lartist'] = urlencode( $currentsong['Artist'] );
                    }
                    if ( ( $currentsong['Title'] != "" ) && ( $currentsong['Title'] != "停止中" ) ) {
                        //					$currentsong['Lsong'] = urlencode( str_replace("/", "／／", $currentsong['Title'] ) );
                        $currentsong['Lsong'] = urlencode( $currentsong['Title'] );
                    }
                }
            } else {
                //	プレイリスト無し
                $currentsong['file'] = $currentsong['filepath'] = "";
                $currentsong['Album'] = $currentsong['Lartist'] = $currentsong['Artist'] = $currentsong['Genre'] = "";
                $currentsong['Title'] = $currentsong['Titledsp'] = "停止中";
                $currentsong['timelength'] = 100;
                $currentsong['timecurrent'] = 0;
                $noplay = true;
            }
            if ( isset($currentstatus['bitrate']) ) {
                list( $rate, $depth, $chnel ) = explode(":",$currentstatus['audio']);
                if ( trim($depth) == "dsd" ) {
                    //	dsd
                    $currentsong['format'] = "DSD";
                    $currentsong['bitrate'] = "";
                    $currentsong['audios'] = "";
                } else {
                    if ($currentstatus['bitrate']=="0") {
                        $currentsong['bitrate'] = "";
                    } else {
                        $currentsong['bitrate'] = number_format($currentstatus['bitrate']) ."kbps";
                    }
                    $ratedata = intval($rate/100)/10 ."kHz / ". $depth ."bit";
                    $currentsong['audios'] = $ratedata ." : ". number_format( $currentstatus['bitrate'] ) ."kbps";
                    $currentsong['format'] = intval($rate/100)/10 ."kHz / ". $depth ."bit";
                }
            } else {
                $currentsong['bitrate'] = "";
                $currentsong['audios'] = "";
                $currentsong['format'] = "";
            }

            if ( isset($currentstatus['state'] ) ) {
                $currentsong['state'] = $currentstatus['state'];
            } else {
                $currentsong['state'] = "";
            }
            if ( $currentstatus['random'] == 1 ) {
                $currentsong['random'] = 1;
            } else {
                $currentsong['random'] = 0;
            }
            $rpl = 0;
            if ( $currentstatus['repeat'] == 1 ) {
                $rpl = 1;
                if ( isset($status['single'] ) && ($status['single'][0]==1) ) $rpl = 2;
            }
            $currentsong['repeatmode'] = $rpl;

            if ( ! isset($currentsong['Composer']) ) $currentsong['Composer'] = "";
            if ( ! isset($currentsong['Date']) ) $currentsong['Date'] = "";

            //	次
            $currentsong['nTitle'] = "";
            $currentsong['nAlbum'] = "";
            $currentsong['nArtist'] = "";
            if ( ! $noplay ) {
                $playlist = $this->Mpd->getPlaylistInfo( $currentsong['Pos']+1 );
                if ( count($playlist) > 0 ) {
                    if ( isset($playlist[0]['Title']) ) {
                        $currentsong['nTitle'] = $playlist[0]['Title'];
                    } else {
                        if ( preg_match( "/http\:\/\//", $playlist[0]['file']) == 1 ) {
                            $currentsong['nTitle'] = "streaming(Internet radio)";
                        } elseif ( isset( $playlist[0]['file']) ) {
                            $wk = explode("/",$playlist[0]['file'] );
                            $currentsong['nTitle'] = array_pop( $wk );
                        } else {
                            $currentsong['nTitle'] = "unknown";
                        }
                    }
                    if ( isset($playlist[0]['Album']) ) {
                        $currentsong['nAlbum'] = $playlist[0]['Album'];
                    } else {
                        $currentsong['nAlbum'] = "";
                    }
                    if ( isset($playlist[0]['Artist']) ) {
                        $currentsong['nArtist'] = $playlist[0]['Artist'];
                    } else {
                        $currentsong['nArtist'] = "";
                    }

                }
            }

            $this->set( 'currentsong', $currentsong );
        } else {
            $currentsong = [ 'error' => true ];
        }

        $this->set( 'currentsong', $currentsong );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'currentsong');
    }

    /*
     *
     */
    public function currentplaylist ( ) {
        $playlist = $this->Mpd->getPlaylistInfo( );
        $currentstatus = $this->Mpd->getStatus( );
        $currentsong = $this->Mpd->getCurrentSong( );

        $outary = array();
        foreach ($playlist as $value) {
            //	ネットラジオ
            if ( preg_match( "/http\:\/\//", $value['file']) == 1 ) {
                $fn = explode("/",$value['file']);
                if ( !isset($value['Title']) ) {
                    $title = array_pop($fn);
                    if ( $title == "" ) $title = "streaming(Internet radio)";
                } else {
                    $title = $value['Title'];
                }
                if ( isset($value['Name'] ) ) {
                    $artist = $value['Name'];
                } else {
                    $artist = "";
                }
            } else {
                //	ファイル群
                $artist = ( isset( $value['Artist'] ) ) ? $value['Artist'] : NULL;
                if ( ! isset($value['Title']) ) {
                    $wk = explode("/",$value['file'] );
                    $title = array_pop( $wk );
                } else {
                    $title = $value['Title'];
                }
            }
            $tm = (isset($value['Time'])) ? Intval( $value['Time'] / 60 ) .":". sprintf('%02d', ( $value['Time'] % 60 ) ) : NULL;
            $cs = ( isset($currentstatus['songid']) && ( $currentstatus['songid']==$value['Id'] ) ) ? 1 : 0;
            $album = ( isset( $value['Album'] ) ) ? $value['Album'] : NULL;
            $track = ( isset( $value['Track'] ) ) ? $value['Track'] : NULL;
            $outary[] = array(
                'file' => $value['file'],
                'Time' => $tm,
                'Title' => $title,
                'Artist' => $artist,
                'Album' => $album,
//                  'Genre' => $value['Genre'],
                'Track' => $track,
//              'Disc' => $value['Disc'],
                'Pos' => $value['Pos'],
                'Id' => $value['Id'],
                'nowplay' => $cs,
            );

        }
        $this->set( 'songs', $outary );
        $pos = -1;
        if ( isset( $currentsong['Pos'] ) ) $pos = $currentsong['Pos'];
        $this->set( 'pos', $pos );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', [ 'songs', 'pos' ] );

    }

    /*
     * $ps: $this->Mpd->getStatus( )の戻り値
     */
    private function _playStauts( $ps ) {

        if ( ! isset( $ps['status']['state'] ) ) return false;
        $ret = false;
        switch ( strtolower($ps['status']['state']) ) {
            case self::ON_PLAY :
            case self::NOW_PAUSE :
            case self::NOT_PLAY :
                $ret = strtolower($ps['status']['state']);
                break;

        }
        return $ret;
    }



    //	$id : 現在のプレイリストの曲番号 0が1曲目
    public function playbtn( ) {	//	再生
        //	再生中でなければ、再生
        $cs = $this->Mpd->getStatus( );
        $ps = $this->_playStauts( $cs );

        if ( ( $ps == self::ON_PLAY ) || ( $ps == self::NOW_PAUSE ) ) {			//	再生中 -> pause, pause中 -> 再生
            $go = $this->Mpd->pause( );
        } elseif ( $ps == self::NOT_PLAY ) {	//	STOP中 -> 再生
            //	停止中なので、その曲の先頭から再開
            $csn = 0;
            if ( isset( $cs['status']['song'] ) ) $csn = $cs['status']['song'];
            $go = $this->Mpd->play( $csn );
        } else {	//	それ以外
            $retval = [ 'status' => false ];
        }

        $ncs = $this->Mpd->getStatus( );
        $nps = $this->_playStauts( $ncs );
        $retval = [ 'status' => $nps ];

        $this->set( 'result', $retval );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }


    //	$id : 現在のプレイリストの曲番号 0が1曲目
    public function play( ) {	//	再生
        //	再生中でなければ、再生
        $cs = $this->Mpd->getStatus( );
        $ps = $this->_playStauts( $cs );

        if ( $ps == self::NOW_PAUSE ) {	//	再生中
            $go = $this->Mpd->play( );
            $retval = [ 'status' => self::ON_PLAY ];
        } elseif ( $ps == self::NOT_PLAY ) {
            //	停止中なので、その曲の先頭から再開
            $csn = 0;
            if ( isset( $cs['status']['song'] ) ) $csn = $cs['status']['song'];
            $go = $this->Mpd->play( $csn );
            $retval = [ 'status' => self::ON_PLAY ];
        } else {	//	それ以外
            $retval = [ 'status' => false ];
        }

        $this->set( 'result', $retval );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    //	再生中は、pause
    //	pause中も、pause
    //	停止中は、play
    public function pause( ) {	//	pause
        //	再生中であればpause
        $cs = $this->Mpd->getStatus( );
        $ps = $this->_playStauts( $cs );

        if ( $ps == self::ON_PLAY ) {	//	再生中
            $go = $this->Mpd->pause( );
            $retval = [ 'status' => self::NOW_PAUSE ];
        } else {
            $retval = [ 'status' => false ];
        }
        $this->set( 'result', $retval );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }


    public function volume( $vol = 0 ) {
        $vol = intval($vol);

        $cstatus = $this->Mpd->getStatus( );
        $currentvol = intval( $cstatus['status']['volume'] );
        if ( ( $currentvol != -1 ) && ( $currentvol != $vol ) && ( $currentvol >= 0 ) && ( $currentvol <= 100 ) ) {
            $pl = $this->Mpd->setVolume( $vol );
        }
        $this->set( 'result', [ 'result' => true ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }



    //	次の曲へ
    public function nextsong( ) {
        $pl = $this->Mpd->nextSong( );
        $this->set( 'result', [ 'result' => true ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    //	前の曲へ
    public function prevsong( ) {
        //	現在曲を再生中なら、その先頭へ
        $cs = $this->Mpd->getStatus( );	//	現在の再生中時刻の取得

        if ( isset( $cs['status']['state'] ) ) {
            if ( ( strtolower( $cs['status']['state'] ) == "play") || ( strtolower( $cs['status']['state'] ) == "pause") ) {
                //	曲の途中の場合は先頭まで
                if ( isset( $cs['status']['time'] ) ) {
                    list( $ct, $lg ) = explode( ":", $cs['status']['time'] );

                    //	先頭から2秒以内なら、前の曲へ
                    if ( $ct > 2 ) {
                        $pl = $this->Mpd->seekId( $cs['status']['songid'], 0 );
                    } else {
                        $pl = $this->Mpd->previousSong( );
                    }
                }
            } else {
                //	再生中ではない
                if ( isset( $cs['status']['song'] ) && ( $cs['status']['song'] > 0 ) ) {
                    //	先頭ではない
                    $pl = $this->Mpd->play( ( $cs['status']['song'] - 1 ) );
                }
            }
            $retval = [ 'result' => true ];
        } else {
            $retval = [ 'result' => false ];
        }
        $this->set( 'result', $retval );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    //	停止
    public function stop( ) {
        $cs = $this->Mpd->getStatus( );
        if ( isset( $cs['status']['state'] ) && ( strtolower( $cs['status']['state'] ) == "stop" ) ) {
            //	停止中なら、現在のプレイリストをクリア
            $go = $this->Mpd->clear( );
        } else {
            //	pauseなら再開
            $go = $this->Mpd->stop( );
        }
        $this->set( 'result', true );
        $this->set( '_serialize', ['result'] );
    }

    public function seekfw( ) {

        $cstatus = $this->Mpd->getStatus( );
        if ( isset( $cstatus['status']['state'] ) && ( strtolower( $cstatus['status']['state'] ) == "play" ) ) {
            //	再生中のみ機能
            list( $ct,$lt ) = explode( ":", $cstatus['status']['time'] );

            $tgt  = $ct + $this->seekTime;		//	シーク後の時間
            if ( $tgt > $lt ) $tgt = $lt;
            if ( $lt != 0 ) {
                //	__SEEKTIME__分スキップ
                $pl = $this->Mpd->seekId( $cstatus['status']['songid'], $tgt );
            }
        }
        $this->set( 'result', [ 'result' => true ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function seekbk( ) {

        $cstatus = $this->Mpd->getStatus( );
        if ( isset( $cstatus['status']['state'] ) && ( strtolower( $cstatus['status']['state'] ) == "play" ) ) {
            //	再生中のみ機能
            list( $ct,$lt ) = explode( ":", $cstatus['status']['time'] );

            $tgt  = $ct - $this->seekTime;		//	シーク後の時間
            if ( $tgt < 0 ) $tgt = 0;
            if ( $lt != 0 ) {
                //	__SEEKTIME__分スキップ
                $pl = $this->Mpd->seekId( $cstatus['status']['songid'], $tgt );
            }
        }
        $this->set( 'result', [ 'result' => true ] );
//		$this->set( '_serialize', ['result'] );
        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function seektime( ) {
        $haveTm = false;
        $tm = $this->request->getQuery( 'tm', 0 );
        if ( $tm > 0 ) $haveTm = true;

        if ( $haveTm ) {
            $cstatus = $this->Mpd->getStatus( );
            $tm = intval($tm);
            if ( isset( $cstatus['status']['state'] ) && ( ( strtolower( $cstatus['status']['state'] ) == "play" ) || ( strtolower( $cstatus['status']['state'] ) == "pause") ) ) {
                //	再生中のみ機能
                list( $ct,$lt ) = explode( ":", $cstatus['status']['time'] );

                if ($tm<=$lt) {
                    $pl = $this->Mpd->seekId( $cstatus['status']['songid'], $tm );
                }
            }
        }
        $this->set( 'result', [ 'result' => true ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function repeat ( ) {
        //	戻り値
        //		0 : repeat無し
        //		1 : 全曲リピート
        //		2 : 1曲リピート

        $pl = $this->Mpd->getStatus( );
        $rp = 0;
        $sg = 0;
        $rpval = 0;
        if ( isset( $pl['status']['random'] ) && ( $pl['status']['repeat'] == 0 ) ) {
            $rp = 1;
            $rpval = 1;
        } else {
            if ( $pl['single'][0] == 0 ) {
                $sg = 1;
                $rp = 1;
                $rpval = 2;
            }
        }
        $this->Mpd->repeat( $rp );
        $this->Mpd->runCommand( 'single', array( $sg ));
        $this->set( 'repeat', [ 'repeat' => $rpval ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'repeat' );
    }

    public function random ( ) {
        //	戻り値
        //		0 : 通常再生
        //		1 : シャッフル再生
        //		2 : シャッフル＆リピート
        $pl = $this->Mpd->getStatus( );
        $sp = 0;
        $rpval = 0;
        if ( isset( $pl['status']['random'] ) && ( $pl['status']['random'] == 0 ) ) {
            $rpval = 1;
            $sp = 1;
            if ( $pl['status']['repeat'] == 1 ) {
                if ( $pl['single'][0] == 1 ) $this->Mpd->runCommand( 'single', array( 0 ));
                $rpval = 2;
            }
        }

        $this->Mpd->random( $sp );
        $this->set( 'random', [ 'repeat' => $rpval ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'random' );
    }


    public function add1song( ) {

        $haveSong = false;

        $plmd = "c";
        $sg1 = urldecode( trim( $this->request->getQuery( 'songfile', "" ) ) );
        if ( strlen($sg1) > 0 ) $haveSong = true;

        $plmd = trim( $this->request->getQuery( 'cl', "c" ) );
        if ( $plmd != "a" ) $plmd = "c";

        $retval = false;
        if ( $haveSong ) {

            $res = $this->Mpd->find( [ 'file' => $sg1 ] );

            $isHave = false;
            foreach ( $res as $one ) {
                if ( strpos( $one['file'] , $sg1 ) !== false ) {
                    $isHave = true;
                }
                if ( $isHave ) break;
            }
            if ( $isHave ) {
                //	指定された曲が存在

                //	cの場合、クリア
                if ( $plmd == "c" ) $go = $this->Mpd->clear( );
                //	プレイリストに追加
                $out = $this->Mpd->addSong($sg1);

                //	再生中でなければ、再生
                $cs = $this->Mpd->getStatus( );
                if ( $cs['status']['state'] == "pause" ) {
                    $go = $this->Mpd->pause( );
                } elseif ( $cs['status']['state'] == "stop" ) {
                    $csn = 0;
                    if (isset($cs['status']['song'])) $csn = $cs['status']['song'];
                    $go = $this->Mpd->play( $csn );
                }
                $retval = true;

            }

        }

        $this->set( 'result', [ 'result' => $retval ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function playradio( ) {
        $nodata = false;
        $sg1 = 0;

        $sg1 = $this->request->getQuery( 'rsid', 0 );
        $plmd = $this->request->getQuery( 'opt', "c" );
        if ( ( $plmd != 'c' ) && ( $plmd != 'a' ) ) $plmd = "c";

        $retval = false;
        if ( $sg1 > 0 ) {
            $radiodata = $this->netradio->get( $sg1 );
            if ( $radiodata !== false ) {

                //	cの場合、クリア
                if ( $plmd == "c" ) $go = $this->Mpd->clear( );
                //	お気に入りリストに追加
                $rdo = $this->Mpd->addSong( $radiodata->radiourl );

                //	再生中でなければ、再生
                $cs = $this->Mpd->getStatus( );
                if ( $cs['status']['state'] == "pause" ) {
                    $go = $this->Mpd->pause( );
                } elseif ( $cs['status']['state'] == "stop" ) {
                    $csn = 0;
                    if (isset($cs['status']['song'])) $csn = $cs['status']['song'];
                    $go = $this->Mpd->play( $csn );
                }
                $retval = true;
            }
        }
        $this->set( 'result', [ 'result' => $retval ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function setoutput ( ) {
        $retval = false;
        $dacno = $this->request->getQuery( 'dacid', 0 );
        $onoff = $this->request->getQuery( 'onoff', 0 );

        $dvlist = $this->_getOutputs( );

        $ret = false;
        if ( isset($dvlist[$dacno]) ) {
            if ( $dvlist[$dacno]['outputenabled'] ==0 ) {
                $ret = $this->_enableOutput($dacno);
            } else {
                $ret = $this->_disableOutput($dacno);
            }
        }
        if ( $ret ) $retval = true;
        $this->set( 'result', [ 'result' => $retval ] );
        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    function _getOutputs( ) {	//	出力デバイスのリストを取得

        $dv = $this->Mpd->getOutputs( );

        $dvlist = array();
        if (count($dv['outputs'])>0) {
            $k = 0;
            for ( $i = 0; $i < count($dv['outputs']); $i = $i + 2 ) {
                $dvlist[ $dv['outputs'][$i]['outputid'] ] = array(
                    'outputid' => $dv['outputs'][$i]['outputid'],
                    'outputname' => $dv['outputs'][$i+1]['outputname'],
                    'outputenabled' => $dv['outputs'][$i+1]['outputenabled'],
                );
                $k ++;
            }
        }

        return($dvlist);
    }

    function _disableOutput( $id = 0 ) {
        $dvlist = $this->_getOutputs();
        if ( isset($dvlist[$id]) ) {
            $dv = $this->Mpd->disableOutput( $id );
            return true;
        } else {
            return false;
        }
    }

    function _enableOutput( $id = 0 ) {
        $dvlist = $this->_getOutputs();
        if ( isset($dvlist[$id]) ) {
            $dv = $this->Mpd->enableOutput( $id );
            return true;
        } else {
            return false;
        }
    }

    public function setlabel( ) {
        $ret = false;
        $albmid = intval( $this->request->getQuery( 'alid', 0 ) );
        $newslbl = intval( $this->request->getQuery( 'lbl', 0 ) );
        Log::write( 'debug', 'setlabel: '. "{$albmid}/{$newslbl}" );
        $lbldata = $this->favolabels->find()->where( [ 'label_key' => $newslbl ] )->first( );

        $albmcount = $this->albums->get( $albmid );
        Log::write( 'debug', 'favolabels: '. print_r( $lbldata, true ) );
        Log::write( 'debug', 'albmcount: '. print_r( $albmcount, true ) );
        if ( ( $albmcount != false ) && ( $lbldata != false ) ) {
            $albmcount->favolabels_key = $newslbl;
            if ( $this->albums->save( $albmcount ) ) {
                $ret = [ "name" => $lbldata->colorname, "color" => $lbldata->colorcode ];
            }
        }

        $this->set( 'result', [ 'result' => $ret ] );
        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    //	$id : 現在のプレイリストの曲番号 0が1曲目
    public function playsongid( ) {	//	再生
        $sid = intval( $this->request->getQuery( 'sid', -1 ) );

        //	現在再生中のプレイリストを取得し、sidが存在すれば、そこへスキップ
        $pl = $this->Mpd->getPlaylistInfo( );
        $st = false;
        if ( count($pl) > 0 ) {
            foreach ($pl as $value) {
                if ( $value['Pos'] == $sid ) {
                    $go = $this->Mpd->play( $sid );
                    $st = true;
                    break;
                }
            }
        }
        if ( $st ) {
            $this->set( 'result', [ 'result' => true ] );
        } else {
            $this->set( 'result', [ 'result' =>  false ] );
        }

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    //	$id : 現在のプレイリストの曲番号 0が1曲目
    public function deletesong( ) {	//	再生
        $sid = intval( $this->request->getQuery( 'sid', -1 ) );

        if ( $sid >= 0 ) {
            $pl = $this->Mpd->deleteSong( $sid );
        }

        $this->set( 'result', [ 'result' => true ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }

    public function saveplaylist( ) {
        $entity = new PlaylistForm();
        $isValid = $entity->validate( $this->request->getData() );
        if ( $isValid ) {
            $lnm = $this->request->getData( 'listname', '');
            $plstall = $this->Mpd->getPlaylists();
            if ( in_array($lnm, $plstall) ) {
                $lnm .= "_". date("YmdHis");	//	同じお気に入りリスト名存在
            }
            //	保存
            $this->Mpd->savePlaylist( $lnm );
            $this->set( 'result', [ 'status' => true ] );

        } else {
            $errs = $entity->getErrors( );
            if ( isset( $errs['listname'] ) ) $msg = array_shift( $errs['listname'] );
            $this->set( 'result', [ 'status' => false, 'errors' => $msg ] );
        }

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }


    public function addplaylist( ) {

        $haveSong = false;
        $playlist = urldecode( trim( $this->request->getQuery( 'playlist', "" ) ) );
        if ( strlen($playlist) > 0 ) $haveSong = true;
        $plmd = trim( $this->request->getQuery( 'cl', "c" ) );
        if ( $plmd != "a" ) $plmd = "c";

        $retval = false;
        if ( $haveSong ) {

            $res = $this->Mpd->getPlaylistDetail( $playlist );
            if ( isset( $res['file'] ) && ( ! empty( $res['file'] ) ) ) {
                //	cの場合、クリア
                if ( $plmd == "c" ) $go = $this->Mpd->clear( );
                //	プレイリストに追加
                $out = $this->Mpd->loadPlaylists( $playlist );

                //	再生中でなければ、再生
                $cs = $this->Mpd->getStatus( );
                if ( $cs['status']['state'] == "pause" ) {
                    $go = $this->Mpd->pause( );
                } elseif ( $cs['status']['state'] == "stop" ) {
                    $csn = 0;
                    if (isset($cs['status']['song'])) $csn = $cs['status']['song'];
                    $go = $this->Mpd->play( $csn );
                }

                $retval = true;
            }

        }

        $this->set( 'result', [ 'result' => $retval ] );

        $this->viewBuilder()
            ->setClassName(JsonView::class)
            ->setOption('serialize', 'result' );
    }


    /*
     *  deleteplaylist method
     *  playlist削除
     */
    public function deleteplaylist( )
    {
        $playlist = urldecode( trim( $this->request->getQuery( 'playlist', "" ) ) );
        $exist = $this->Mpd->getPlaylistDetail( $playlist );
        if ( isset( $exist['file'] ) && ( ! empty( $exist['file'] ) ) ) {
            //  play list が存在する
            $del = $this->Mpd->deletePlaylist( $playlist );
            $retval = true;
        } else {
            $retval = false;
        }

        return $this->redirect( [ 'controller'=> 'mpdapi', 'action'=>'index' ] );

    }


    /*
     *  notice method
     */
    public function notice( )
    {

    }



}
