<?php
/*

	Takeo Akiyama, Fourthgate Inc.

	mpcbridge Ver. 6.1 / 2021.04.03

*/
declare(strict_types=1);

namespace App\Controller;

use App\Controller\AppController;
use Cake\Event\Event;
use Cake\Core\Configure;
use Cake\ORM\TableRegistry;
use App\Controller\Component\RequestHandler;
use Cake\Network\Request;
use Cake\Log\Log;

use App\Form\SearchForm;

class SearchController extends AppController
{

    private $coverUrl;
    private $alphabet;
    private $alphabetgrid;
    private $genres;
    private $songs;
    private $albums;
    private $favolabels;

    private $searchArtistMethod;
    private $searchSongMethod;
    private $searchFormat;

    private $doc_title;
    private $session;

    public function initialize(): void
    {
        parent::initialize();

        $this->session = $this->getRequest()->getSession();

        $this->albums = TableRegistry::getTableLocator()->get('Albums');
        $this->songs = TableRegistry::getTableLocator()->get('Songs');
        $this->favolabels = TableRegistry::getTableLocator()->get('Favolabels');
        $this->genres = TableRegistry::getTableLocator()->get('Genres');

        $this->coverUrl = Configure::read('CoverUrl');

        $this->searchArtistMethod = Configure::read('Search.artist');
        $this->searchSongMethod = Configure::read('Search.song');
        $this->searchFormat = Configure::read('Search.format');

        $this->doc_title = "検索";
    }

    public function beforeRender(\Cake\Event\EventInterface $event) {
        parent::beforeRender($event);

        $this->set( 'cover_url', $this->coverUrl );
        $this->set( 's_artist', $this->searchArtistMethod );
        $this->set( 's_song', $this->searchSongMethod );
        $this->set( 's_format', $this->searchFormat );

        $labels = $this->_labels( );
        $this->set( 'labelcolor', $labels[0] );
        $this->set( 'labelname', $labels[1] );
        $this->set( 'title', $this->doc_title );
    }

    public function index( ) {
        $entity = new SearchForm();
        $postData = null;

        $genres = $this->genres->find('Sorted')->all( );
        $all_genres = [ ];
        if ( $genres->count( ) > 0 ) {
            foreach ( $genres as $one ) {
                if ( $one->genre != '' ) $all_genres[ $one->genre ] = $one->genre;
            }
        }

        if ( $this->request->is( [ 'post', 'put' ] ) ) {
            $postData = $this->request->getData();
//            Log::write( 'debug', "Search index post data: ". print_r($postData,true) );
            $isValid1 = $entity->validate( $postData );
            $isValid2 = true;

            if ( ( $postData['artist_name'] == "" ) && ( $postData['song_title'] == "" ) && ( $postData['composer'] == "" ) ) {
                $entity->setErrors(["artist_name" => ["_required" => "アーティスト名/曲名/作曲者名のいずれかを入力してください"]]);
                $isValid2 = false;
            }

            if ( $isValid1 && $isValid2 ) {
                $this->session->write( 'search', $postData );
                return $this->redirect( [ 'action' => 'result' ] );
            }

            $this->set( 'valid1', $isValid1 );
            $this->set( 'valid2', $isValid2 );
        } else {
            if ( $this->session->check('search') ) {
                $postData = $this->session->read('search');
            }
        }
        $this->set( 'all_genres', $all_genres );
        $this->set( 'entity', $entity );
        $this->set( 'data', $postData );
        $this->set( 'title', "フリーワード検索" );

    }

    public function result( ) {
        if ( $this->session->check('search') ) {
            $data = $this->session->read('search');
        } else {
            return $this->redirect( [ 'action' => 'index' ] );
            $data = [ ];
        }

        if (
            ( ! ( isset( $data['composer'] ) && isset( $data['artist_name'] ) && isset( $data['song_title'] ) && isset( $data['song_year'] ) && isset( $data['sortodr'] ) && isset( $data['artist_method'] ) && isset( $data['song_method'] ) ) ) ||
            ( ( $data['composer'] == "" ) && ( $data['artist_name'] == "" ) && ( $data['song_title'] == "" ) && ( $data['song_year'] == "" ) && ( $data['song_format'] == "" ) )
        ) {
            $this->Flash->set( "検索結果条件を入力してください" );
            return $this->redirect( [ 'action' => 'index' ] );
        }

//        Log::write( 'debug', "Search data: ". print_r($data,true) );
        $all_songs = $this->songs->find( 'Search', $data )->all( );
        $number = $all_songs->count();
        if ( $number > 300 ) {
            $this->Flash->set( "検索結果が300件以上あります。もっと絞り込んでください。" );
            return $this->redirect( [ 'action' => 'index' ] );
        }

        $all_albums = $this->albums->find( 'Search', $data )->all( );

        $this->set( 'allsong', $all_songs );
        $this->set( 'all_albumdata', $all_albums );
        $this->set( 'data', $data );
        $this->set( 'title', "フリーワード検索結果" );
    }

    public function playall( ) {

        $retval = false;
        if ($this->request->is('post')) {
            $entity = new SearchForm();
            $data = $this->request->getData();
//            Log::write( 'debug', "playall data: ". print_r($data,true) );
            $isValid1 = $entity->validate( $data );
            $isValid2 = true;

            if (
                ( ! ( isset( $data['composer'] ) && isset( $data['artist_name'] ) && isset( $data['song_title'] ) && isset( $data['song_year'] ) && isset( $data['sortodr'] ) && isset( $data['artist_method'] ) && isset( $data['song_method'] ) ) ) ||
                ( ( $data['composer'] == "" ) && ( $data['artist_name'] == "" ) && ( $data['song_title'] == "" ) && ( $data['song_year'] == "" ) )
            ) {
                $isValid2 = false;
            }

            if ( $isValid1 && $isValid2 ) {
//                Log::write( 'debug', "playall start..." );
                $query_s = $this->songs->find( 'Search', $data );
                $number = $query_s->count();

                if ( ( $number < 100 ) && ( $number > 0 ) ) {
                    $retval = true;
                    $all_songs = $query_s->all( );
                    $play_on = false;
                    if ( isset( $data['adtyp'] ) && ( $data['adtyp'] == "c" ) ) {
                        $this->Mpd->clear( );
                        $play_on = true;
                    }

                    foreach ( $all_songs as $one) {
                        $res = $this->Mpd->find( [ 'file' => $one->file ] );
                        if ( is_array( $res ) ) {
                            $res_song = array_shift( $res );
                            if ( (isset( $res_song['file'])) && ( $res_song['file'] == $one->file ) ) {
                                //	プレイリストに追加
                                $out = $this->Mpd->addSong( $one->file );
                            }
                        }
                    }
                    if ( $play_on ) {
                        $this->Mpd->play( );
                    }

                }

            }

        }
        $this->set( 'result', $retval );
//        $this->set( '_serialize', ['result'] );
        $this->viewBuilder()->setOption('serialize', [ 'result', ]);
    }

    function _labels ( ) {

        $lblall = $this->favolabels->find( )->order( [ 'Favolabels.id' => 'ASC' ] )->all( );

        if ( ! empty( $lblall ) ) {
            $a1 = $a2 = array();
            foreach ( $lblall as $value ) {
                $a1[ $value->id ] = $value->colorcode;
                $a2[ $value->id ] = $value->colorname;
            }
            $retval = array(
                0 => $a1,
                1 => $a2
            );
        } else {
            $retval = array(
                0 => array( 1 => "808080" ),
                1 => array( 1 => "ラベルなし" )
            );
        }
        return $retval;
    }

}
