<?php

namespace App\Form;

use Cake\Form\Form;
use Cake\Form\Schema;
use Cake\Validation\Validator;
use Cake\Core\Configure;

class SearchForm extends Form
{

    protected function _buildSchema(Schema $schema) : Schema
    {
        $schema->addField( 'artist_name', ['type' => 'string'] );
        $schema->addField( 'song_title', ['type' => 'string'] );
        $schema->addField( 'composer', ['type' => 'string'] );
        $schema->addField( 'artist_method', ['type' => 'string'] );
        $schema->addField( 'song_method', ['type' => 'string'] );
        $schema->addField( 'song_year', ['type' => 'string'] );
        $schema->addField( 'song_format', ['type' => 'string'] );

        return $schema;
    }

    public function validationDefault(Validator $validator) : Validator
    {
        $validator->allowEmptyString( 'artist_name' );
        $validator->allowEmptyString( 'song_title' );
        $validator->allowEmptyString( 'composer' );
        $validator->allowEmptyString( 'song_year' );

        $validator
            ->allowEmptyString(
                'artist_method'
            )
            ->add(	'artist_method',
                'valid',
                [
                    'rule' => function($value, $data){
                        $tp = Configure::read('Search.artist');
                        // 集計単位？
                        if ( isset( $tp[$value] ) ) return true;
                        return false;
                    },
                    'message' => '不正な選択値です'
                ]
            );

        $validator
            ->allowEmptyString( 'song_method' )
            ->add(	'song_method',
                'valid',
                [
                    'rule' => function($value, $data){
                        $tp = Configure::read('Search.song');
                        // 集計単位？
                        if ( isset( $tp[$value] ) ) return true;
                        return false;
                    },
                    'message' => '不正な選択値です'
                ]
            );

        $validator->add( 'song_year', 'num', [
            'rule' => [ 'numeric' ],
            'message' => '数値を入力してください'
        ]);
        $validator->add( 'song_year', 'num2', [
            'rule' => [ 'range', 0, 3000 ],
            'message' => '西暦年を入力してください'
        ]);
        $validator->add( 'artist_name', 'length1', [
            'rule' => [ 'maxLength', 100 ],
            'message' => '100文字以下で入力してください'
        ]);
        $validator->add( 'song_title', 'length1', [
            'rule' => [ 'maxLength', 100 ],
            'message' => '100文字以下で入力してください'
        ]);
        $validator->add( 'composer', 'length1', [
            'rule' => [ 'maxLength', 100 ],
            'message' => '100文字以下で入力してください'
        ]);
        $validator
            ->allowEmptyString( 'song_format' )
            ->add(	'song_format',
                'valid',
                [
                    'rule' => function($value, $data){
                        $tp = Configure::read('Search.format');
                        // 集計単位？
                        if ( isset( $tp[$value] ) ) return true;
                        return false;
                    },
                    'message' => '不正な選択値です'
                ]
            );
        return $validator;
    }

    /*
        public function setErrors($errors) {
            $this->_errors = $errors;
        }
    */

    protected function _execute(array $data) : bool
    {
        return true;
    }

}
