<?php
/*

	Takeo Akiyama, Fourthgate Inc.

	mpcbridge Ver. 6.1 / 2021.04.03

*/
declare(strict_types=1);

namespace App\Model\Table;

use Cake\Core\Configure;
use Cake\Database\Expression\QueryExpression;
use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\Validation\Validator;

/**
 * Albums Model
 *
 * @method \App\Model\Entity\Album newEmptyEntity()
 * @method \App\Model\Entity\Album newEntity(array $data, array $options = [])
 * @method \App\Model\Entity\Album[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Album get($primaryKey, $options = [])
 * @method \App\Model\Entity\Album findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \App\Model\Entity\Album patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Album[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \App\Model\Entity\Album|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Album saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Album[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Album[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \App\Model\Entity\Album[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Album[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class AlbumsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('albums');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior( 'Timestamp' );
        $this->addBehavior(
            'CounterCache',
            [
                'Favolabels' => [ 'albums_count' ]
            ]
        );
        $this->belongsTo(
            'Favolabels',
            [
                'bindingKey' => 'label_key',
                'foreignKey' => 'favolabels_key'
            ]
        );
        $this->hasMany(
            'Songs',
            [
                'className' => 'Songs',
                'foreignKey' => 'albums_id',
                'sort' => [ 'Songs.track' => 'ASC' ]
            ]
        );
    }
    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator
    {

        $validator
            ->nonNegativeInteger('favolabels_key')
            ->requirePresence('favolabels_key', 'create')
            ->notEmptyString('favolabels_key');

        $validator
            ->scalar('albumtitle')
            ->allowEmptyString('albumtitle');

        $validator
            ->scalar('albumtitle_sort')
            ->allowEmptyString('albumtitle_sort');

        $validator
            ->scalar('artist')
            ->allowEmptyString('artist');

        $validator
            ->scalar('artist_sort')
            ->allowEmptyString('artist_sort');

        $validator
            ->scalar('genre')
            ->allowEmptyString('genre');

        $validator
            ->scalar('path')
            ->allowEmptyString('path', 'ファイルパスは必須です', false );

        $validator
            ->date('release_date')
            ->allowEmptyDate('release_date');

        $validator
            ->integer('songs_count')
            ->allowEmptyString('songs_count');

        $validator
            ->scalar('musicbrainz_albumid')
            ->maxLength('musicbrainz_albumid', 40)
            ->allowEmptyString('musicbrainz_albumid');

        return $validator;
    }

    //  検索
    public function findSearch(\Cake\ORM\Query $query, array $options)
    {
        $query
            ->matching('Songs')
            ->contain(['Songs','Favolabels'])
            ->select(['Albums.id', 'Albums.albumtitle', 'Albums.path', 'Albums.favolabels_key', 'Favolabels.colorname']);
        //  ソート順
        $order = "";
        if (isset($options['sortodr'])) {
            $order = trim($options['sortodr']);
        }
        //  検索条件
        $astyle = $style = 0;
        if (isset($options['artist_method'])) {
            $astyle = abs(intval($options['artist_method']));
        }
        if (isset($options['song_method'])) {
            $style = abs(intval($options['song_method']));
        }
        //  演奏者名
        if (isset($options['artist_name']) && $options['artist_name'] != "") {
            $aname = trim($options['artist_name']);
            $y_aname = mb_convert_kana( trim($options['artist_name']) , "KCVa");
            if ($astyle == 2) {
                //	後方一致
                $query->where(['OR' => ['Songs.artist LIKE' => "%" . $aname, 'Songs.artist_sort LIKE' => "%" . $y_aname, ]]);
            } elseif ($astyle == 1) {
                //	前方一致
                $query->where(['OR' => ['Songs.artist LIKE' => $aname . "%", 'Songs.artist_sort LIKE' => $y_aname . "%", ]]);
            } else {
                //	部分一致
                $query->where(['OR' => ['Songs.artist LIKE' => "%" . $aname . "%", 'Songs.artist_sort LIKE' => "%" . $y_aname . "%", ]]);
            }
        }
        //  曲名
        if (isset($options['song_title']) && $options['song_title'] != "") {
            if ($style == 2) {
                //	後方一致
                $query->where(['Songs.title LIKE' => "%" . $options['song_title']]);
            }
            if ($style == 1) {
                //	前方一致
                $query->where(['Songs.title LIKE' => $options['song_title'] . "%"]);
            } else {
                //	部分一致
                $query->where(['Songs.title LIKE' => "%" . $options['song_title'] . "%"]);
            }
        }
        //  ジャンル
        if (isset($options['genre']) && $options['genre'] != "") {
            $query->where( ['Songs.genre' => $options['genre'] ] );
        }
        //  年度
        if (isset($options['song_year']) && $options['song_year'] != "") {
            $query->where(['Songs.songdate LIKE' => $options['song_year'] . "%"]);
        }
        //	作曲者
        if (isset($options['composer']) && $options['composer'] != "") {
            $query->where(['Songs.composer LIKE' => "%" . $options['composer'] . "%"]);
        }
        //  サウンドフォーマット
        if (isset($options['song_format']) && $options['song_format'] != "") {
            $song_format = intval( $options['song_format'] );
            $tp = Configure::read('Search.format');
            // 集計単位？
            if ( ! isset( $tp[$song_format] ) ) $song_format = 0;
            if ($song_format == 1 ) {
                //	CDフォーマット
                $query
                    ->where( [ 'Songs.sampling_frequency' => 44.1, 'Songs.quantization_bits' => 16 ] );
            } elseif ( $song_format == 2 ) {
                //	ハイレゾ
                $query
                    ->where(function (QueryExpression $exp, Query $q) {
                        return $exp
                            ->gt( 'Songs.sampling_frequency', 44.1 )
                            ->gt( 'Songs.quantization_bits', 16 );
                    });
            }
        }

        //	ソート
        switch ($order) {
            case "s1":
                //	年代降順
                $query->order(["IF(Albums.release_date = NULL, '0000', Albums.release_date) DESC, Albums.albumtitle ASC"]);
                break;
            case "s2":
                //	アルファベット昇順
                $query->order(['Albums.albumtitle' => "ASC", 'Albums.release_date' => 'ASC']);
                break;
            case "s3":
                //	アルファベット降順
                $query->order(['Albums.albumtitle' => "DESC", 'Albums.release_date' => 'ASC']);
                break;
            default:
                //	年代昇順
                $query->order(["IF(Albums.release_date = NULL, '0000', Albums.release_date) ASC, Albums.albumtitle ASC"]);
                break;
        }
        //  グルーピング
        $query->group(['Albums.id']);
        return $query;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
//        $rules->add($rules->existsIn(['label_key'], 'Favolabels'));
        return $rules;
    }
}
