<?php
declare(strict_types=1);

namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Cake\ORM\Behavior\CounterCacheBehavior;

/**
 * Favolabels Model
 *
 * @method \App\Model\Entity\Favolabel newEmptyEntity()
 * @method \App\Model\Entity\Favolabel newEntity(array $data, array $options = [])
 * @method \App\Model\Entity\Favolabel[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Favolabel get($primaryKey, $options = [])
 * @method \App\Model\Entity\Favolabel findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \App\Model\Entity\Favolabel patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Favolabel[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \App\Model\Entity\Favolabel|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Favolabel saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Favolabel[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Favolabel[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \App\Model\Entity\Favolabel[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Favolabel[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class FavolabelsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);
        $this->setTable('favolabels');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');
        $this->addBehavior('Timestamp');
        $this->hasMany(
            'Albums',
            [
                'className' => 'Albums',
                'bindingKey' => 'label_key',
                'foreignKey' => 'favolabels_key',
                'sort' => [ 'Albums.artist_yomisrt' => 'ASC' ]
            ]
        );
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator

    {

        $validator
            ->allowEmptyString('colorcode')
            ->add('colorcode',
                'custom',
                ['rule' => function ($value, $context) {
                    return (bool) preg_match('/^#[0-9a-f]{3,6}$/i', $value
                    );
                },
                    'message' => 'カラーコードが不正です'
                ]
            );

        $validator
            ->allowEmptyString('colorname', 'ラベル名を入力してください', false )
            ->add(
                'colorname',
                'valid',
                [
                    'rule' => ['maxLength', 100],
                    'message' => 'ラベル名は100文字以内で入力してください'
                ]
            );

        $validator
            ->integer('albums_count')
            ->allowEmptyString('albums_count');

        $validator
            ->nonNegativeInteger('label_key')
            ->requirePresence('label_key', 'create')
            ->notEmptyString('label_key')
            ->add(
                'label_key',
                'unique',
                ['rule' => 'validateUnique', 'provider' => 'table']
            );

        return $validator;
    }
}
