<?php
/*

	Takeo Akiyama, Fourthgate Inc.

	mpcbridge Ver. 6.1 / 2021.04.03

*/
declare(strict_types=1);

namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\ORM\Behavior\CounterCacheBehavior;
use Cake\Database\Expression\QueryExpression;
use Cake\Validation\Validator;
use Cake\Core\Configure;
use Cake\Log\Log;

/**
 * Songs Model
 *
 * @property \App\Model\Table\AlbumsTable&\Cake\ORM\Association\BelongsTo $Albums
 *
 * @method \App\Model\Entity\Song newEmptyEntity()
 * @method \App\Model\Entity\Song newEntity(array $data, array $options = [])
 * @method \App\Model\Entity\Song[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Song get($primaryKey, $options = [])
 * @method \App\Model\Entity\Song findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \App\Model\Entity\Song patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Song[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \App\Model\Entity\Song|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Song saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Song[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Song[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \App\Model\Entity\Song[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Song[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */

class SongsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('songs');
        $this->setDisplayField('title');
        $this->setPrimaryKey('id');

        $this->addBehavior(
            'Timestamp',
            [
                'events' => [
                    'Model.beforeSave' => [
                        'created' => 'new',
                        'modified' => 'always'
                    ]
                ]
            ]
        );
        $this->addBehavior(
            'CounterCache',
            [
                'Albums' => [ 'songs_count' ]
            ]
        );
        $this->belongsTo(
            'Albums',
            [
                'foreignKey' => 'albums_id',
                'joinType' => 'INNER'
            ]
        );
    }
    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator->integer('id')->allowEmptyString('id', 'create');

        $validator
            ->scalar('file')
            ->allowEmptyFile('file', 'ファイルは必須です', false );

        $validator
            ->scalar('path')
            ->allowEmptyString('path', 'ファイルパスは必須です', false );

        $validator
            ->scalar('title')
            ->allowEmptyString('title');

        $validator
            ->integer('length')
            ->allowEmptyString('length');

        $validator
            ->scalar('artist')
            ->allowEmptyString('artist');

        $validator
            ->scalar('artist_sort')
            ->allowEmptyString('artist_sort');

        $validator
            ->scalar('composer')
            ->allowEmptyString('composer');

        $validator
            ->scalar('composer_sort')
            ->allowEmptyString('composer_sort');

        $validator
            ->scalar('genre')
            ->allowEmptyString('genre');

        $validator
            ->integer('track')
            ->allowEmptyString('track');

        $validator
            ->date('songdate')
            ->allowEmptyDate('songdate');

        $validator
            ->scalar('albumtitle')
            ->allowEmptyString('albumtitle');

        $validator
            ->scalar('albumtitle_sort')
            ->allowEmptyString('albumtitle_sort');

        $validator
            ->scalar('album_artist')
            ->allowEmptyString('album_artist');

        $validator
            ->scalar('album_artist_sort')
            ->allowEmptyString('album_artist_sort');

        $validator
            ->scalar('fileformat')
            ->maxLength('fileformat', 20)
            ->allowEmptyFile('fileformat');

        $validator
            ->dateTime('last_modified')
            ->allowEmptyDateTime('last_modified');

        $validator
            ->scalar('performer')
            ->maxLength('performer', 250)
            ->allowEmptyString('performer');

        $validator
            ->scalar('conductor')
            ->maxLength('conductor', 250)
            ->allowEmptyString('conductor');

        $validator
            ->scalar('label')
            ->maxLength('label', 250)
            ->allowEmptyString('label');

        $validator
            ->scalar('musicbrainz_artistid')
            ->maxLength('musicbrainz_artistid', 40)
            ->allowEmptyString('musicbrainz_artistid');

        $validator
            ->scalar('musicbrainz_albumid')
            ->maxLength('musicbrainz_albumid', 40)
            ->allowEmptyString('musicbrainz_albumid');

        $validator
            ->scalar('musicbrainz_albumartistid')
            ->maxLength('musicbrainz_albumartistid', 40)
            ->allowEmptyString('musicbrainz_albumartistid');

        $validator
            ->scalar('musicbrainz_trackid')
            ->maxLength('musicbrainz_trackid', 40)
            ->allowEmptyString('musicbrainz_trackid');

        $validator
            ->scalar('musicbrainz_releasetrackid')
            ->maxLength('musicbrainz_releasetrackid', 40)
            ->allowEmptyString('musicbrainz_releasetrackid');

        $validator
            ->scalar('musicbrainz_workid')
            ->maxLength('musicbrainz_workid', 40)
            ->allowEmptyString('musicbrainz_workid');

        $validator
            ->decimal('sampling_frequency')
            ->allowEmptyString('sampling_frequency');

        $validator
            ->allowEmptyString('quantization_bits');

        $validator
            ->decimal('channels')
            ->allowEmptyString('channels');

        $validator
            ->scalar('dsd_rate')
            ->maxLength('dsd_rate', 10)
            ->allowEmptyString('dsd_rate');

        return $validator;
    }


    //  検索
    public function findSearch(\Cake\ORM\Query $query, array $options)
    {
        //  ソート順
        $order = "";
        if (isset($options['sortodr'])) {
            $order = trim($options['sortodr']);
        }
        //  検索条件
        $astyle = $style = 0;
        if (isset($options['artist_method'])) {
            $astyle = abs(intval($options['artist_method']));
        }
        if (isset($options['song_method'])) {
            $style = abs(intval($options['song_method']));
        }
        //  演奏者名
        if (isset($options['artist_name']) && $options['artist_name'] != "") {
            $aname = trim($options['artist_name']);
            $aname_the = preg_replace( "/^(The\s)/i", "", $aname );
            $y_aname = mb_convert_kana($aname, "KCVa");
            if ($astyle == 2) {
                //	後方一致
                $query->where(['OR' => ['Songs.artist LIKE' => "%" . $aname, 'Songs.artist_sort LIKE' => "%" . $y_aname, ]]);
            } elseif ($astyle == 1) {
                //	前方一致
                $query->where(['OR' => ['Songs.artist LIKE' => $aname . "%", 'Songs.artist_sort LIKE' => $y_aname . "%", ]]);
            } else {
                //	部分一致
                $query->where(['OR' => ['Songs.artist LIKE' => "%" . $aname . "%", 'Songs.artist_sort LIKE' => "%" . $y_aname . "%", ]]);
            }
        }
        //  曲名
        if (isset($options['song_title']) && $options['song_title'] != "") {
            if ($style == 2) {
                //	後方一致
                $query->where(['Songs.title LIKE' => "%" . $options['song_title']]);
            }
            if ($style == 1) {
                //	前方一致
                $query->where(['Songs.title LIKE' => $options['song_title'] . "%"]);
            } else {
                //	部分一致
                $query->where(['Songs.title LIKE' => "%" . $options['song_title'] . "%"]);
            }
        }
        //  ジャンル
        if (isset($options['genre']) && $options['genre'] != "") {
            $query->where( ['Songs.genre' => $options['genre'] ] );
        }

        //  年度
        if (isset($options['song_year']) && $options['song_year'] != "") {
            $query->where(['Songs.songdate LIKE' => $options['song_year'] . "%"]);
        }
        //	作曲者
        if (isset($options['composer']) && $options['composer'] != "") {
            $query->where(['Songs.composer LIKE' => "%" . $options['composer'] . "%"]);
        }
        //  音声フォーマット
        if (isset($options['song_format']) && $options['song_format'] != "") {
            $song_format = intval( $options['song_format'] );
            $tp = Configure::read('Search.format');
            // 集計単位？
            if ( ! isset( $tp[$song_format] ) ) $song_format = 0;
            if ($song_format == 1 ) {
                //	CDフォーマット
                $query
                    ->where( [ 'Songs.sampling_frequency' => 44.1, 'Songs.quantization_bits' => 16 ] );
            } elseif ( $song_format == 2 ) {
                //	ハイレゾ
                $query
                    ->where(function (QueryExpression $exp, Query $q) {
                        return $exp
                            ->gt( 'Songs.sampling_frequency', 44.1 )
                            ->gt( 'Songs.quantization_bits', 16 );
                    });
            }
        }

        //	ソート
        switch ($order) {
            case "s1":
                //	年代降順
                $query->order(["IF(Songs.songdate = NULL, '0000', Songs.songdate) DESC, Songs.albums_id ASC, Songs.track ASC"]);
                break;
            case "s2":
                //	アルファベット昇順
                $query->order(['Songs.title' => "ASC", 'Songs.albums_id' => 'ASC', 'Songs.track' => 'ASC']);
                break;
            case "s3":
                //	アルファベット降順
                $query->order(['Songs.title' => "DESC", 'Songs.albums_id' => 'ASC', 'Songs.track' => 'ASC']);
                break;
            default:
                //	年代昇順
                $query->order(["IF(Songs.songdate = NULL, '0000', Songs.songdate) ASC, Songs.albums_id ASC, Songs.track ASC"]);
                break;
        }
        return $query;
    }
    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
//        $rules->add($rules->existsIn(['albums_id'], 'Albums'));
        return $rules;
    }
}
